@@ -135,13 +135,13 @@ int tkimg_Read(
 	char *dst /* where to put the result */,
 	int count /* number of bytes */
 ) {
-    register int i, c;
+    int i, c;
     int bytesRead, bytesToRead;
     char *dstPtr;
 
     switch (handle->state) {
       case IMG_STRING:
-	if (count > handle->length) {
+	if ((unsigned int)count > handle->length) {
 	    count = handle->length;
 	}
 	if (count) {
@@ -223,7 +223,8 @@ size_t tkimg_Read2(
 	char *dst /* where to put the result */,
 	size_t count /* number of bytes */
 ) {
-    register int i, c;
+    size_t i;
+    int c;
     size_t bytesRead, bytesToRead;
     char *dstPtr;
 
@@ -261,7 +262,7 @@ size_t tkimg_Read2(
 	   	if (bufEnd < 0)
 		    return bufEnd;
 	    }
-	    if (bufStart + bytesToRead <= bufEnd +1) {
+	    if (bufStart + (int)bytesToRead <= bufEnd + 1) {
 #ifdef DEBUG_LOCAL
 		    printf("All in buffer: memcpy %d bytes\n", bytesToRead);
 #endif
@@ -381,7 +382,7 @@ int tkimg_Write(
     const char *src /* where to get the data */,
     int count /* number of bytes */
 ) {
-    register int i;
+    int i;
     int curcount, bufcount;
 
     if (handle->state == IMG_CHAN) {
@@ -432,7 +433,7 @@ size_t tkimg_Write2(
     bufcount = curcount + count + count/3 + count/52 + 1024;
 
     /* make sure that the DString contains enough space */
-    if (bufcount >= (handle->buffer->spaceAvl)) {
+    if (bufcount >= (size_t)(handle->buffer->spaceAvl)) {
 	Tcl_DStringSetLength(handle->buffer, bufcount + 4096);
 	handle->data = Tcl_DStringValue(handle->buffer) + curcount;
     }
@@ -472,8 +473,8 @@ static char const base64_table[64] = {
 };
 
 int tkimg_Putc(
-    register int c /* character to be written */,
-    register tkimg_MFile *handle /* handle containing decoder data and state */
+    int c /* character to be written */,
+    tkimg_MFile *handle /* handle containing decoder data and state */
 ) {
     /* In fact, here should be checked first if the dynamic
      * string contains enough space for the next character.
