@@ -8,7 +8,7 @@
  *
  * The following format options are available:
  *
- * Write BMP image: "ico -resolution <list>"
+ * Write BMP image: "bmp -resolution <list>"
  *
  * -resolution <list>: Set the resolution property of the output file.
  *                     The default is 74 dpi (no option)
@@ -29,6 +29,8 @@
 #include <math.h>
 #include "init.c"
 
+/* #define DEBUG_LOCAL 1 */
+
 /* Compression types */
 #define BI_RGB          0
 #define BI_RLE8         1
@@ -70,11 +72,11 @@ static void putint(tkimg_MFile *handle, int i);
 static int
 ChnMatch(
     Tcl_Channel chan,
-    const char *fileName,
-    Tcl_Obj *format,
+    TCL_UNUSED(const char *),
+    TCL_UNUSED(Tcl_Obj *),
     int *widthPtr,
     int *heightPtr,
-    Tcl_Interp *interp
+    TCL_UNUSED(Tcl_Interp *)
 ) {
     tkimg_MFile handle;
 
@@ -88,14 +90,14 @@ ChnMatch(
 static int
 ObjMatch(
     Tcl_Obj *data,
-    Tcl_Obj *format,
+    TCL_UNUSED(Tcl_Obj *),
     int *widthPtr,
     int *heightPtr,
-    Tcl_Interp *interp
+    TCL_UNUSED(Tcl_Interp *)
 ) {
     tkimg_MFile handle;
 
-    if (!tkimg_ReadInit(data, 'B', &handle)) {
+    if (! tkimg_ReadInit(data, 'B', &handle)) {
         return 0;
     }
     return CommonMatch(&handle, widthPtr, heightPtr,
@@ -106,8 +108,8 @@ static int
 ChnRead(
     Tcl_Interp *interp,
     Tcl_Channel chan,
-    const char *fileName,
-    Tcl_Obj *format,
+    TCL_UNUSED(const char *),
+    TCL_UNUSED(Tcl_Obj *),
     Tk_PhotoHandle imageHandle,
     int destX, int destY,
     int width, int height,
@@ -126,7 +128,7 @@ static int
 ObjRead(
     Tcl_Interp *interp,
     Tcl_Obj *data,
-    Tcl_Obj *format,
+    TCL_UNUSED(Tcl_Obj *),
     Tk_PhotoHandle imageHandle,
     int destX, int destY,
     int width, int height,
@@ -134,7 +136,9 @@ ObjRead(
 ) {
     tkimg_MFile handle;
 
-    tkimg_ReadInit(data,'B',&handle);
+    if (! tkimg_ReadInit(data, 'B', &handle)) {
+        return TCL_ERROR;
+    }
     return CommonRead(interp, &handle, imageHandle, destX, destY,
             width, height, srcX, srcY);
 }
@@ -190,7 +194,7 @@ static int StringWrite(
 static unsigned int
 getUInt32(unsigned char *buf)
 {
-    return (buf[0] | buf[1] << 8 | buf[2] << 16 | buf[3] << 24);
+    return (buf[0] | buf[1] << 8 | buf[2] << 16 | (unsigned int)buf[3] << 24);
 }
 
 static unsigned int
@@ -248,11 +252,11 @@ CommonMatch(
         return 0;
     }
 
-    offBits = (buf[11]<<24) + (buf[10]<<16) + (buf[9]<<8) + buf[8];
+    offBits = ((unsigned int)buf[11]<<24) + (buf[10]<<16) + (buf[9]<<8) + buf[8];
     c = buf[12];
     if ((c == 40) || (c == 64)) {
-        *widthPtr = (buf[19]<<24) + (buf[18]<<16) + (buf[17]<<8) + buf[16];
-        *heightPtr = (buf[23]<<24) + (buf[22]<<16) + (buf[21]<<8) + buf[20];
+        *widthPtr = ((unsigned int)buf[19]<<24) + (buf[18]<<16) + (buf[17]<<8) + buf[16];
+        *heightPtr = ((unsigned int)buf[23]<<24) + (buf[22]<<16) + (buf[21]<<8) + buf[20];
         if (tkimg_Read2(handle, (char *) buf, 24) != 24) {
             return 0;
         }
@@ -269,12 +273,16 @@ CommonMatch(
     } else {
         return 0;
     }
-    if (*widthPtr <= 0 || *heightPtr <= 0)
+    if (*widthPtr <= 0 || *heightPtr <= 0) {
         return 0;
+    }
+    if (nBits <= 0) {
+        return 0;
+    }
 
     if (colorMap) {
         if (c > 36) {
-            if (tkimg_Read2(handle, (char *) buf, c - 36) != c - 36 )
+            if (tkimg_Read2(handle, (char *) buf, c - 36) != (size_t)c - 36 )
                 return 0;
         }
         if (compression == BI_BITFIELDS) {
@@ -294,12 +302,17 @@ CommonMatch(
         if (nBits<16) {
             unsigned char colbuf[4], *ptr;
             offBits -= (3+(c!=12)) * clrUsed;
-            *colorMap = ptr = (unsigned char *) ckalloc(3*clrUsed);
+            *colorMap = ptr = (unsigned char *) attemptckalloc(3*clrUsed);
+            if (ptr == NULL) {
+                return 0;
+            }
             for (i = 0; i < clrUsed; i++) {
                 if (tkimg_Read2(handle, (char *) colbuf, 3+(c!=12)) != 3+(c!=12))
                     return 0;
                 *ptr++ = colbuf[0]; *ptr++ = colbuf[1]; *ptr++ = colbuf[2];
-                /*printf("color %d: %d %d %d\n", i, colbuf[2], colbuf[1], colbuf[0]);*/
+#ifdef DEBUG_LOCAL
+                printf("color %d: %d %d %d\n", i, colbuf[2], colbuf[1], colbuf[0]);
+#endif
             }
         }
         while (offBits>28) {
@@ -308,7 +321,7 @@ CommonMatch(
                 return 0;
         }
         if (offBits > 0) {
-            if (tkimg_Read2(handle, (char *) buf, offBits) != offBits)
+            if (tkimg_Read2(handle, (char *) buf, offBits) != (size_t)offBits)
                 return 0;
         }
         if (numCols) {
@@ -334,17 +347,19 @@ CommonRead(
     int srcX, int srcY
 ) {
     Tk_PhotoImageBlock block;
-    int numBits, bytesPerLine, numCols, comp, x, y;
+    int numBits, bytesPerLine, numCols = 0, comp, x, y;
     int fileWidth, fileHeight;
     unsigned char *colorMap = NULL;
     unsigned int intMask[3];
     BitmapChannel masks[3];
-    char buf[10];
+    char buf[20];
     unsigned char *line = NULL, *expline = NULL;
     unsigned short rgb;
 
-    CommonMatch(handle, &fileWidth, &fileHeight, &colorMap, &numBits,
-                 &numCols, &comp, intMask);
+    if (CommonMatch(handle, &fileWidth, &fileHeight, &colorMap, &numBits,
+                    &numCols, &comp, intMask) == 0) {
+        goto error;
+    }
 
     if (numBits == 16) {
         if (comp == BI_BITFIELDS) {
@@ -362,7 +377,9 @@ CommonRead(
         }
     }
 
-    /* printf("reading %d-bit BMP %dx%d\n", numBits, width, height); */
+#ifdef DEBUG_LOCAL
+    printf("reading %d-bit BMP %dx%d\n", numBits, width, height);
+#endif
     if (comp == BI_RLE8 || comp == BI_RLE4) {
         tkimg_ReadBuffer(1);
     }
@@ -373,9 +390,9 @@ CommonRead(
 
     bytesPerLine = ((numBits * fileWidth + 31)/32)*4;
 
-    /* printf("bytesPerLine = %d numBits=%d (%dx%d)\n", 
-     *        bytesPerLine, numBits, width, height);
-     */ 
+#ifdef DEBUG_LOCAL
+    printf("bytesPerLine = %d numBits=%d (%dx%d)\n", bytesPerLine, numBits, width, height);
+#endif
     block.pixelSize = 3;
     block.pitch = width * 3;
     block.width = width;
@@ -386,15 +403,29 @@ CommonRead(
     block.offset[3] = block.offset[0];
 
     if (comp == BI_RGB || comp == BI_BITFIELDS) {       /* No compression */
-        line = (unsigned char *) ckalloc(bytesPerLine);
+        line = (unsigned char *) attemptckalloc(bytesPerLine);
+        if (line == NULL) {
+            Tcl_AppendResult (interp, "Unable to allocate memory for image data.", (char *) NULL);
+            goto error;
+        }
         for(y=srcY+height; y<fileHeight; y++) {
-            tkimg_Read2(handle, (char *)line, bytesPerLine);
+            if (bytesPerLine != tkimg_Read2(handle, (char *)line, bytesPerLine)) {
+                Tcl_AppendResult (interp, "Unable to read pixel row.", (char *) NULL);
+                goto error;
+            }
         }
         switch (numBits) {
             case 32:
-                block.pixelPtr = expline = (unsigned char *) ckalloc(3*width);
+                block.pixelPtr = expline = (unsigned char *) attemptckalloc(3*width);
+                if (expline == NULL) {
+                    Tcl_AppendResult (interp, "Unable to allocate memory for image data.", (char *) NULL);
+                    goto error;
+                }
                 for( y = height-1; y>=0; y--) {
-                    tkimg_Read2(handle, (char *)line, bytesPerLine);
+                    if (bytesPerLine != tkimg_Read2(handle, (char *)line, bytesPerLine)) {
+                        Tcl_AppendResult (interp, "Unable to read pixel row.", (char *) NULL);
+                        goto error;
+                    }
                     for (x = srcX; x < (srcX+width); x++) {
                         *expline++ = line[x*4 + 0];
                         *expline++ = line[x*4 + 1];
@@ -410,7 +441,10 @@ CommonRead(
             case 24:
                 block.pixelPtr = line + srcX*3;
                 for( y = height-1; y>=0; y--) {
-                    tkimg_Read2(handle, (char *)line, bytesPerLine);
+                    if (bytesPerLine != tkimg_Read2(handle, (char *)line, bytesPerLine)) {
+                        Tcl_AppendResult (interp, "Unable to read pixel row.", (char *) NULL);
+                        goto error;
+                    }
                     if (tkimg_PhotoPutBlock(interp, imageHandle, &block,
                             destX, destY+y, width, 1, TK_PHOTO_COMPOSITE_SET) == TCL_ERROR) {
                         goto error;
@@ -418,9 +452,16 @@ CommonRead(
                 }
                 break;
             case 16:
-                block.pixelPtr = expline = (unsigned char *) ckalloc(3*width);
+                block.pixelPtr = expline = (unsigned char *) attemptckalloc(3*width);
+                if (expline == NULL) {
+                    Tcl_AppendResult (interp, "Unable to allocate memory for image data.", (char *) NULL);
+                    goto error;
+                }
                 for( y = height-1; y>=0; y--) {
-                    tkimg_Read2(handle, (char *)line, bytesPerLine);
+                    if (bytesPerLine != tkimg_Read2(handle, (char *)line, bytesPerLine)) {
+                        Tcl_AppendResult (interp, "Unable to read pixel row.", (char *) NULL);
+                        goto error;
+                    }
                     for (x = srcX; x < (srcX+width); x++) {
                         rgb = getUInt16 (&line[x*2]);
                         *expline++ = ((rgb & masks[2].mask) >> masks[2].shiftin) << masks[2].shiftout;
@@ -435,11 +476,22 @@ CommonRead(
                 }
                 break;
             case 8:
-                block.pixelPtr = expline = (unsigned char *) ckalloc(3*width);
+                block.pixelPtr = expline = (unsigned char *) attemptckalloc(3*width);
+                if (expline == NULL) {
+                    Tcl_AppendResult (interp, "Unable to allocate memory for image data.", (char *) NULL);
+                    goto error;
+                }
                 for( y = height-1; y>=0; y--) {
-                    tkimg_Read2(handle, (char *)line, bytesPerLine);
+                    if (bytesPerLine != tkimg_Read2(handle, (char *)line, bytesPerLine)) {
+                        Tcl_AppendResult (interp, "Unable to read pixel row.", (char *) NULL);
+                        goto error;
+                    }
                     for (x = srcX; x < (srcX+width); x++) {
-                        memcpy(expline, colorMap+(3*line[x]),3);
+                        if (line[x] > numCols) {
+                            Tcl_AppendResult(interp, "Color index exceeds color map size", (char *) NULL);
+                            goto error;
+                        }
+                        memcpy(expline, colorMap + 3*line[x], 3);
                         expline += 3;
                     }
                     if (tkimg_PhotoPutBlock(interp, imageHandle, &block,
@@ -450,17 +502,28 @@ CommonRead(
                 }
                 break;
             case 4:
-                block.pixelPtr = expline = (unsigned char *) ckalloc(3*width);
+                block.pixelPtr = expline = (unsigned char *) attemptckalloc(3*width);
+                if (expline == NULL) {
+                    Tcl_AppendResult (interp, "Unable to allocate memory for image data.", (char *) NULL);
+                    goto error;
+                }
                 for( y = height-1; y>=0; y--) {
                     int c;
-                    tkimg_Read2(handle, (char *)line, bytesPerLine);
+                    if (bytesPerLine != tkimg_Read2(handle, (char *)line, bytesPerLine)) {
+                        Tcl_AppendResult (interp, "Unable to read pixel row.", (char *) NULL);
+                        goto error;
+                    }
                     for (x = srcX; x < (srcX+width); x++) {
                         if (x&1) {
                             c = line[x/2] & 0x0f;
                         } else {
                             c = line[x/2] >> 4;
                         }
-                        memcpy(expline, colorMap+(3*c),3);
+                        if (c > numCols) {
+                            Tcl_AppendResult(interp, "Color index exceeds color map size", (char *) NULL);
+                            goto error;
+                        }
+                        memcpy(expline, colorMap + 3*c, 3);
                         expline += 3;
                     }
                     if (tkimg_PhotoPutBlock(interp, imageHandle, &block,
@@ -471,13 +534,24 @@ CommonRead(
                 }
                 break;
             case 1:
-                block.pixelPtr = expline = (unsigned char *) ckalloc(3*width);
+                block.pixelPtr = expline = (unsigned char *) attemptckalloc(3*width);
+                if (expline == NULL) {
+                    Tcl_AppendResult (interp, "Unable to allocate memory for image data.", (char *) NULL);
+                    goto error;
+                }
                 for( y = height-1; y>=0; y--) {
                     int c;
-                    tkimg_Read2(handle, (char *)line, bytesPerLine);
+                    if (bytesPerLine != tkimg_Read2(handle, (char *)line, bytesPerLine)) {
+                        Tcl_AppendResult (interp, "Unable to read pixel row.", (char *) NULL);
+                        goto error;
+                    }
                     for (x = srcX; x < (srcX+width); x++) {
                         c = (line[x/8] >> (7-(x%8))) & 1;
-                        memcpy(expline, colorMap+(3*c),3);
+                        if (c > numCols) {
+                            Tcl_AppendResult(interp, "Color index exceeds color map size", (char *) NULL);
+                            goto error;
+                        }
+                        memcpy(expline, colorMap + 3*c, 3);
                         expline += 3;
                     }
                     if (tkimg_PhotoPutBlock(interp, imageHandle, &block,
@@ -488,7 +562,7 @@ CommonRead(
                 }
                 break;
             default:
-                sprintf(buf, "%d", numBits);
+                tkimg_snprintf(buf, 20, "%d", numBits);
                 Tcl_AppendResult(interp, buf,
                         "-bits BMP file not (yet) supported", (char *) NULL);
                 goto error;
@@ -502,10 +576,14 @@ CommonRead(
 
         x = srcX;
         y = fileHeight - 1;
-        block.pixelPtr = expline = (unsigned char *) ckalloc (3*fileWidth);
+        block.pixelPtr = expline = (unsigned char *) attemptckalloc (3*fileWidth);
+        if (expline == NULL) {
+            Tcl_AppendResult (interp, "Unable to allocate memory for image data.", (char *) NULL);
+            goto error;
+        }
 
         if (numBits != 4 && numBits != 8) {
-            sprintf (buf, "%d", numBits);
+            tkimg_snprintf (buf, 20, "%d", numBits);
             Tcl_AppendResult(interp, "RLE compression not supported for ",
                               buf, "-bit pixel values", (char *) NULL);
             goto error;
@@ -516,13 +594,22 @@ CommonRead(
                 Tcl_AppendResult(interp, "Unexpected EOF", (char *) NULL);
                 goto error;
             }
-            /* printf("In: (%d %X) --> \n", rleBuf[0], rleBuf[1]); */
+#ifdef DEBUG_LOCAL
+            printf("In: (%d %X) --> \n", rleBuf[0], rleBuf[1]);
+#endif
             if (rleBuf[0] != 0) {
                 howMuch = rleBuf[0];
                 switch (numBits) {
                     case 8: {
                         for (i=0; i<howMuch; i++, x++) {
+                            if (rleBuf[1] > numCols) {
+                                Tcl_AppendResult(interp, "Color index exceeds color map size", (char *) NULL);
+                                goto error;
+                            }
                             memcpy (expline, colorMap + 3*rleBuf[1], 3);
+                            if (x>=srcX+width-1) {
+                                break;
+                            }
                             expline += 3;
                         }
                         break;
@@ -534,6 +621,10 @@ CommonRead(
                             } else {
                                 c = rleBuf[1] >> 4;
                             }
+                            if (c > numCols) {
+                                Tcl_AppendResult(interp, "Color index exceeds color map size", (char *) NULL);
+                                goto error;
+                            }
                             memcpy(expline, colorMap + 3*c, 3);
                             if (x>=srcX+width-1) {
                                 break;
@@ -547,7 +638,9 @@ CommonRead(
                 if (rleBuf[1]>2) {
                     /* uncompressed record */
                     howMuch = rleBuf[1];
-                    /* printf ("Uncompressed: %d\n", howMuch); fflush (stdout); */
+#ifdef DEBUG_LOCAL
+                    printf ("Uncompressed: %d\n", howMuch); fflush (stdout);
+#endif
                     switch (numBits) {
                         case 8: {
                             for (i=0; i<howMuch; i++, x++) {
@@ -555,7 +648,14 @@ CommonRead(
                                     Tcl_AppendResult(interp, "Unexpected EOF", (char *)NULL);
                                     goto error;
                                 }
+                                if (val > numCols) {
+                                    Tcl_AppendResult(interp, "Color index exceeds color map size", (char *) NULL);
+                                    goto error;
+                                }
                                 memcpy(expline, colorMap + 3*val, 3);
+                                if (x>=srcX+width-1) {
+                                    break;
+                                }
                                 expline += 3;
                             }
                             break;
@@ -572,6 +672,10 @@ CommonRead(
                                     } else {
                                         c = val >> 4;
                                     }
+                                    if (c > numCols) {
+                                        Tcl_AppendResult(interp, "Color index exceeds color map size", (char *) NULL);
+                                        goto error;
+                                    }
                                     memcpy(expline, colorMap + 3*c, 3);
                                     if (x>=srcX+width-1) {
                                         break;
@@ -595,7 +699,13 @@ CommonRead(
                     }
                 } else if (rleBuf[1]==0) {
                     /* End of line */
-                    /* printf("New line: y=%d x=%d\n", y, x); fflush(stdout); */
+#ifdef DEBUG_LOCAL
+                    printf("New line: y=%d x=%d\n", y, x); fflush(stdout);
+#endif
+                    if (destY+y > fileHeight) {
+                        Tcl_AppendResult (interp, "Decoding inconsistency", (char *)NULL);
+                        goto error;
+                    }
                     if (tkimg_PhotoPutBlock(interp, imageHandle, &block, destX, destY+y,
                             width, 1, TK_PHOTO_COMPOSITE_SET) == TCL_ERROR) {
                         goto error;
@@ -603,12 +713,18 @@ CommonRead(
                     y--;
                     x = srcX;
                     expline = block.pixelPtr;
+                    if (y < 0) {
+                        Tcl_AppendResult (interp, "Decoding inconsistency", (char *)NULL);
+                        goto error;
+                    }
                 } else if (rleBuf[1]==1) {
                     /* End of bitmap */
                     break;
                 } else if (rleBuf[1]==2) {
                     /* Deltarecord */
-                    /* printf("Deltarecord\n"); fflush(stdout); */
+#ifdef DEBUG_LOCAL
+                    printf("Deltarecord\n"); fflush(stdout);
+#endif
                     if (2 != tkimg_Read2(handle, (char *) rleDelta, 2)) {
                         Tcl_AppendResult(interp, "Unexpected EOF", (char *) NULL);
                         goto error;
@@ -630,7 +746,7 @@ CommonRead(
     if (expline) {
         ckfree((char *) block.pixelPtr);
     }
-    return TCL_OK ;
+    return TCL_OK;
 
 error:
     tkimg_ReadBuffer(0);
@@ -660,7 +776,7 @@ CommonWrite(
     int resX=75*39, resY=75*39;
     int objc = 0;
     Tcl_Obj **objv = NULL;
- 
+
     /* Decode resolution parameter -resolution List */
     if (tkimg_ListObjGetElements(interp, format, &objc, &objv) != TCL_OK) {
         return TCL_ERROR;
