@@ -197,38 +197,31 @@ static void dtedClose (DTEDFILE *tf)
     return;
 }
 
-/* Read 2 bytes, representing a signed 16 bit integer in the form
-   <LowByte, HighByte>, from a file and convert them into the current
-   machine's format. */
+/*  Read 2 bytes representing a signed 16-bit integer stored in big-endian order. */
 
 static Boln readShort (tkimg_MFile *handle, Short *s)
 {
-    char buf[2];
-    if (2 != tkimg_Read2(handle, buf, 2))
+    unsigned char buf[2];
+    if (2 != tkimg_Read2(handle, (char *)buf, 2))
         return FALSE;
-    *s = (buf[0] & 0xFF) | (buf[1] << 8);
+    *s = (buf[0] << 8) | buf[1];
     return TRUE;
 }
 
-/* Read 4 bytes, representing a signed 32 bit integer in the form
-   <LowByte, HighByte>, from a file and convert them into the current
-   machine's format. */
+/* Read 4 bytes representing a signed 32-bit integer stored in big-endian order. */
 
 static Boln readInt (tkimg_MFile *handle, Int *i)
 {
-    char buf[4];
-    if (4 != tkimg_Read2(handle, buf, 4))
+    unsigned char buf[4];
+    if (4 != tkimg_Read2(handle, (char *)buf, 4))
         return FALSE;
-    *i = ((((Int)buf[0] & 0x000000FFU) << 24) | \
-          (((Int)buf[1] & 0x0000FF00U) <<  8) | \
-          (((Int)buf[2] & 0x00FF0000U) >>  8) | \
-          (((Int)buf[3] & 0x0000FF00U) >> 24));
+    *i = ((unsigned int)buf[0] << 24) | (buf[1] << 16) | (buf[2] << 8) | buf[3];
     return TRUE;
 }
 
 #define OUT Tcl_WriteChars (outChan, str, -1)
 static void printImgInfo (DTEDHEADER *th, FMTOPT *opts,
-                          const char *filename, const char *msg)
+                          TCL_UNUSED(const char *), const char *msg)
 {
     Tcl_Channel outChan;
     char str[256];
@@ -237,22 +230,22 @@ static void printImgInfo (DTEDHEADER *th, FMTOPT *opts,
     if (!outChan) {
         return;
     }
-    sprintf (str, "%s\n", msg);                                              OUT;
-    sprintf (str, "\tLongitude of origin  : %.8s\n", th->uhl.origin_long);   OUT;
-    sprintf (str, "\tLatitude of origin   : %.8s\n", th->uhl.origin_lat);    OUT;
-    sprintf (str, "\tEast-West interval   : %.4s\n", th->uhl.ew_interval);   OUT;
-    sprintf (str, "\tNorth-South interval : %.4s\n", th->uhl.ns_interval);   OUT;
-    sprintf (str, "\tVertical accuracy    : %.4s\n", th->uhl.accuracy);      OUT;
-    sprintf (str, "\tSecurity Code        : %.3s\n", th->uhl.security);      OUT;
-    sprintf (str, "\tDTED level           : %.5s\n", th->dsi.level);         OUT;
-    sprintf (str, "\tNumber of rows       : %.4s\n", th->dsi.rows);          OUT;
-    sprintf (str, "\tNumber of columns    : %.4s\n", th->dsi.cols);          OUT;
-    sprintf (str, "\tCell coverage        : %.2s\n", th->dsi.cell_coverage); OUT;
-    sprintf (str, "\tGamma correction     : %f\n", opts->gamma);             OUT;
-    sprintf (str, "\tMinimum map value    : %d\n", opts->minVal);            OUT;
-    sprintf (str, "\tMaximum map value    : %d\n", opts->maxVal);            OUT;
-    sprintf (str, "\tHost byte order      : %s\n", tkimg_IsIntel ()?
-                                                   strIntel: strMotorola);   OUT;
+    tkimg_snprintf (str, 256, "%s\n", msg);                                              OUT;
+    tkimg_snprintf (str, 256, "\tLongitude of origin  : %.8s\n", th->uhl.origin_long);   OUT;
+    tkimg_snprintf (str, 256, "\tLatitude of origin   : %.8s\n", th->uhl.origin_lat);    OUT;
+    tkimg_snprintf (str, 256, "\tEast-West interval   : %.4s\n", th->uhl.ew_interval);   OUT;
+    tkimg_snprintf (str, 256, "\tNorth-South interval : %.4s\n", th->uhl.ns_interval);   OUT;
+    tkimg_snprintf (str, 256, "\tVertical accuracy    : %.4s\n", th->uhl.accuracy);      OUT;
+    tkimg_snprintf (str, 256, "\tSecurity Code        : %.3s\n", th->uhl.security);      OUT;
+    tkimg_snprintf (str, 256, "\tDTED level           : %.5s\n", th->dsi.level);         OUT;
+    tkimg_snprintf (str, 256, "\tNumber of rows       : %.4s\n", th->dsi.rows);          OUT;
+    tkimg_snprintf (str, 256, "\tNumber of columns    : %.4s\n", th->dsi.cols);          OUT;
+    tkimg_snprintf (str, 256, "\tCell coverage        : %.2s\n", th->dsi.cell_coverage); OUT;
+    tkimg_snprintf (str, 256, "\tGamma correction     : %f\n", opts->gamma);             OUT;
+    tkimg_snprintf (str, 256, "\tMinimum map value    : %d\n", opts->minVal);            OUT;
+    tkimg_snprintf (str, 256, "\tMaximum map value    : %d\n", opts->maxVal);            OUT;
+    tkimg_snprintf (str, 256, "\tHost byte order      : %s\n", tkimg_IsIntel ()?
+                                                               strIntel: strMotorola);   OUT;
     Tcl_Flush (outChan);
 }
 #undef OUT
@@ -270,14 +263,15 @@ static Boln readHeader (tkimg_MFile *handle, DTEDHEADER *th)
     return TRUE;
 }
 
-static Boln readDtedColumn (tkimg_MFile *handle, Short *pixels, Int nRows,
-                            Int nCols, Int curCol, char *buf, Boln hostIsIntel)
+static Boln readDtedColumn (Tcl_Interp *interp, tkimg_MFile *handle, Short *pixels,
+                            Int nRows, Int nCols, Int curCol, char *buf, Boln hostIsIntel)
 {
     Int   i, nBytes;
     Short *mPtr;
     char  *bufPtr = buf;
     Short meridian, parallel;
     Int   block_count;
+    UByte recognition_sentinel;
     UByte *cp;
     Int  checksum, checksum1 = 0;
 
@@ -285,7 +279,7 @@ static Boln readDtedColumn (tkimg_MFile *handle, Short *pixels, Int nRows,
     if (!readInt   (handle, &block_count) ||
         !readShort (handle, &meridian) ||
         !readShort (handle, &parallel)) {
-        printf ("Error reading column header\n");
+        Tcl_AppendResult (interp, "Error reading column header.", (char *) NULL);
         return FALSE;
     }
 
@@ -297,57 +291,61 @@ static Boln readDtedColumn (tkimg_MFile *handle, Short *pixels, Int nRows,
     cp = (UByte *) &parallel;
     checksum1 += cp[0] + cp[1];
 
-    if (hostIsIntel) {
-        block_count = block_count & 0x00ffffff;
-    } else {
-        block_count = (block_count & 0xffffff00) >> 8;
+    recognition_sentinel = (unsigned int)block_count >> 24;
+    block_count &= 0x00ffffff;
+
+    if (recognition_sentinel != 170) {
+        char msg[100];
+        tkimg_snprintf(msg, sizeof(msg),
+                       "Invalid column recognition sentinel 0x%02hhX.",
+                       recognition_sentinel);
+        Tcl_AppendResult (interp, msg, (char *) NULL);
+        return FALSE;
     }
 
     /* Read the elevation data into the supplied column buffer "buf". */
     nBytes = sizeof (Short) * nRows;
-    if (nBytes != tkimg_Read2(handle, buf, nBytes)) {
-        printf ("Error reading elevation data\n");
+    if ((size_t)nBytes != tkimg_Read2(handle, buf, nBytes)) {
+        Tcl_AppendResult (interp, "Error reading elevation data.", (char *) NULL);
         return FALSE;
     }
 
     /* Copy (and swap bytes, if needed) from the column buffer into the
-       pixel array (shorts) . */
+       pixel array (shorts). Simultaneously calculate checksum, part 2. */
     if (hostIsIntel) {
         for (i=0; i<nRows; i++) {
             mPtr = pixels + (i * nCols) + curCol;
-            ((char *)mPtr)[0] = bufPtr[1];
-            ((char *)mPtr)[1] = bufPtr[0];
+            checksum1 += ((unsigned char *)mPtr)[0] = bufPtr[1];
+            checksum1 += ((unsigned char *)mPtr)[1] = bufPtr[0];
             bufPtr += sizeof (Short);
         }
     } else {
         for (i=0; i<nRows; i++) {
             mPtr = pixels + (i * nCols) + curCol;
-            ((char *)mPtr)[0] = bufPtr[0];
-            ((char *)mPtr)[1] = bufPtr[1];
+            checksum1 += ((unsigned char *)mPtr)[0] = bufPtr[0];
+            checksum1 += ((unsigned char *)mPtr)[1] = bufPtr[1];
             bufPtr += sizeof (Short);
         }
     }
 
     /* Read the checksum */
     if (!readInt (handle, &checksum)) {
-        printf ("Error reading checksum\n");
+        Tcl_AppendResult (interp, "Error reading checksum.", (char *) NULL);
         return FALSE;
     }
 
-    /* Calculate checksum, part 2. OPA TODO Incorrect  */
-    cp = (UByte *) pixels;
-    for (i=0; i<nRows*2; i++, cp++) {
-        checksum1 += *cp;
-    }
-
     if (checksum != checksum1) {
-        /* printf ("DTED Checksum Error (%d vs. %d).\n", checksum, checksum1); */
-        /* return FALSE; */
+        char msg[100];
+        tkimg_snprintf(msg, sizeof(msg),
+            "DTED Checksum Error (0x%08x vs. 0x%08x).", checksum, checksum1);
+        Tcl_AppendResult (interp, msg, (char *) NULL);
+        return FALSE;
     }
     return TRUE;
 }
 
-static Boln readDtedFile (tkimg_MFile *handle, Short *buf, Int width, Int height,
+static Boln readDtedFile (Tcl_Interp *interp, tkimg_MFile *handle,
+                          Short *buf, Int width, Int height,
                           Int nchan, Boln hostIsIntel, Boln verbose,
                           Short minVals[], Short maxVals[])
 {
@@ -363,12 +361,16 @@ static Boln readDtedFile (tkimg_MFile *handle, Short *buf, Int width, Int height
         minVals[c] =  MAX_SHORT;
         maxVals[c] =  MIN_SHORT;
     }
-    colBuf = ckalloc (sizeof (Short) * nchan * height);
+    colBuf = (char *)attemptckalloc (sizeof (Short) * nchan * height);
+    if (colBuf == NULL) {
+        Tcl_AppendResult (interp, "Unable to allocate memory for image data.", (char *) NULL);
+        return FALSE;
+    }
 
     /* Read the elevation data column by column. */
     for (x=0; x<width; x++) {
-        if (!readDtedColumn (handle, buf, height, width,
-                             x, colBuf, hostIsIntel)) {
+        if (! readDtedColumn (interp, handle, buf, height, width,
+                              x, colBuf, hostIsIntel)) {
             return FALSE;
         }
     }
@@ -488,7 +490,7 @@ static int ParseFormatOpts(
         return TCL_ERROR;
     if (objc) {
         for (i=1; i<objc; i++) {
-            if (Tcl_GetIndexFromObj (interp, objv[i], (const char *CONST86 *)dtedOptions,
+            if (Tcl_GetIndexFromObj (interp, objv[i], (const char * const *)dtedOptions,
                     "format option", 0, &index) != TCL_OK) {
                 return TCL_ERROR;
             }
@@ -544,7 +546,7 @@ static int ParseFormatOpts(
 
 static int ChnMatch(
     Tcl_Channel chan,
-    const char *filename,
+    TCL_UNUSED(const char *),
     Tcl_Obj *format,
     int *widthPtr,
     int *heightPtr,
@@ -582,6 +584,8 @@ static int CommonMatch(
     DTEDHEADER th;
     FMTOPT opts;
     Int nRows, nCols;
+    Byte buf[5];
+    buf[4] = '\0';
 
     if (ParseFormatOpts (interp, format, &opts) == TCL_ERROR) {
         return FALSE;
@@ -590,8 +594,11 @@ static int CommonMatch(
     if (!readHeader (handle, &th)) {
         return FALSE;
     }
-    sscanf (th.dsi.rows, "%4d", &nRows);
-    sscanf (th.dsi.cols, "%4d", &nCols);
+
+    memcpy (buf, th.dsi.rows, 4);
+    sscanf (buf, "%04d", &nRows);
+    memcpy (buf, th.dsi.cols, 4);
+    sscanf (buf, "%04d", &nCols);
     *widthPtr  = nCols;
     *heightPtr = nRows;
     if (dtedHeaderPtr) {
@@ -662,7 +669,7 @@ static int CommonRead(
     Int matte = 0;
     UByte *pixbufPtr;
     Short *rawbufPtr;
-    Float gtable[IMG_GAMMA_TABLE_SIZE];
+    Double gtable[IMG_GAMMA_TABLE_SIZE];
     Int nchan = 1;
 
     memset (&tf, 0, sizeof (DTEDFILE));
@@ -690,15 +697,21 @@ static int CommonRead(
     }
     if ((outWidth <= 0) || (outHeight <= 0)
         || (srcX >= fileWidth) || (srcY >= fileHeight)) {
-        return TCL_OK;
+        Tcl_AppendResult(interp, "Width or height are negative", (char *) NULL);
+        return TCL_ERROR;
     }
 
     hostIsIntel = tkimg_IsIntel ();
 
-    tf.rawbuf = (Short *)ckalloc (fileWidth * fileHeight *
-                                  nchan * sizeof (Short));
-    readDtedFile (handle, tf.rawbuf, fileWidth, fileHeight, nchan,
-                  hostIsIntel, opts.verbose, minVals, maxVals);
+    tf.rawbuf = (Short *)attemptckalloc (fileWidth * fileHeight * nchan * sizeof (Short));
+    if (tf.rawbuf == NULL) {
+        Tcl_AppendResult (interp, "Unable to allocate memory for image data.", (char *) NULL);
+        return TCL_ERROR;
+    }
+    if (!readDtedFile (interp, handle, tf.rawbuf, fileWidth, fileHeight, nchan,
+                       hostIsIntel, opts.verbose, minVals, maxVals)) {
+        return TCL_ERROR;
+    }
     if (opts.minVal != 0 || opts.maxVal != 0) {
         for (c=0; c<nchan; c++) {
             minVals[c] = opts.minVal;
@@ -712,7 +725,11 @@ static int CommonRead(
         return TCL_ERROR;
     }
 
-    tf.pixbuf = (UByte *) ckalloc (fileWidth * nchan);
+    tf.pixbuf = (UByte *) attemptckalloc (fileWidth * nchan);
+    if (tf.pixbuf == NULL) {
+        Tcl_AppendResult (interp, "Unable to allocate memory for image data.", (char *) NULL);
+        return TCL_ERROR;
+    }
 
     block.pixelSize = nchan;
     block.pitch = fileWidth * nchan;
@@ -748,19 +765,19 @@ static int CommonRead(
 
 static int ChnWrite(
     Tcl_Interp *interp,
-    const char *filename,
-    Tcl_Obj *format,
-    Tk_PhotoImageBlock *blockPtr
+    TCL_UNUSED(const char *),
+    TCL_UNUSED(Tcl_Obj *),
+    TCL_UNUSED(Tk_PhotoImageBlock *)
 ) {
-    Tcl_AppendResult (interp, "Writing not supported for format ", sImageFormat.name, NULL);
+    Tcl_AppendResult (interp, "Writing not supported for format ", sImageFormat.name, (char *)NULL);
     return TCL_ERROR;
 }
 
 static int StringWrite(
     Tcl_Interp *interp,
-    Tcl_Obj *format,
-    Tk_PhotoImageBlock *blockPtr
+    TCL_UNUSED(Tcl_Obj *),
+    TCL_UNUSED(Tk_PhotoImageBlock *)
 ) {
-    Tcl_AppendResult (interp, "Writing not supported for format ", sImageFormat.name, NULL);
+    Tcl_AppendResult (interp, "Writing not supported for format ", sImageFormat.name, (char *)NULL);
     return TCL_ERROR;
 }
