@@ -50,7 +50,7 @@
  * -max <float>:        Specify the maximum pixel value to be used for mapping
  *                      the input data to 8-bit image values.
  *                      Default is the maximum value found in the image data.
- * -scanorder <string>: Specify the scanline order of the input image. Convention 
+ * -scanorder <string>: Specify the scanline order of the input image. Convention
  *                      is storing scan lines from top to bottom.
  *                      Possible values: "TopDown" or "BottomUp".
  * -ascii <bool>:       If set to true, file is written in PPM ASCII format (P3).
@@ -79,6 +79,7 @@
  */
 
 #include <stdlib.h>
+#include <float.h>
 #include <math.h>
 
 /*
@@ -146,7 +147,7 @@ typedef struct {
 } PPMFILE;
 
 #define OUT Tcl_WriteChars (outChan, str, -1)
-static void printImgInfo (int width, int height, int maxVal, int isAscii, int nChans, 
+static void printImgInfo (int width, int height, int maxVal, int isAscii, int nChans,
                           FMTOPT *opts, const char *filename, const char *msg)
 {
     Tcl_Channel outChan;
@@ -156,17 +157,17 @@ static void printImgInfo (int width, int height, int maxVal, int isAscii, int nC
     if (!outChan) {
         return;
     }
-    sprintf (str, "%s %s\n", msg, filename);                                        OUT;
-    sprintf (str, "\tSize in pixel    : %d x %d\n", width, height);                 OUT;
-    sprintf (str, "\tMaximum value    : %d\n", maxVal);                             OUT;
-    sprintf (str, "\tNo. of channels  : %d\n", nChans);                             OUT;
-    sprintf (str, "\tGamma correction : %f\n", opts->gamma);                        OUT;
-    sprintf (str, "\tMinimum map value: %f\n", opts->minVal);                       OUT;
-    sprintf (str, "\tMaximum map value: %f\n", opts->maxVal);                       OUT;
-    sprintf (str, "\tVertical encoding: %s\n", opts->scanOrder == TOP_DOWN?
-                                               strTopDown: strBottomUp);            OUT;
-    sprintf (str, "\tAscii format     : %s\n", isAscii?  "Yes": "No");              OUT;
-    sprintf (str, "\tHost byte order  : %s\n", tkimg_IsIntel ()?  strIntel: strMotorola); OUT;
+    tkimg_snprintf (str, 256, "%s %s\n", msg, filename);                                        OUT;
+    tkimg_snprintf (str, 256, "\tSize in pixel    : %d x %d\n", width, height);                 OUT;
+    tkimg_snprintf (str, 256, "\tMaximum value    : %d\n", maxVal);                             OUT;
+    tkimg_snprintf (str, 256, "\tNo. of channels  : %d\n", nChans);                             OUT;
+    tkimg_snprintf (str, 256, "\tGamma correction : %f\n", opts->gamma);                        OUT;
+    tkimg_snprintf (str, 256, "\tMinimum map value: %f\n", opts->minVal);                       OUT;
+    tkimg_snprintf (str, 256, "\tMaximum map value: %f\n", opts->maxVal);                       OUT;
+    tkimg_snprintf (str, 256, "\tVertical encoding: %s\n", opts->scanOrder == TOP_DOWN?
+                                                           strTopDown: strBottomUp);            OUT;
+    tkimg_snprintf (str, 256, "\tAscii format     : %s\n", isAscii?  "Yes": "No");              OUT;
+    tkimg_snprintf (str, 256, "\tHost byte order  : %s\n", tkimg_IsIntel ()?  strIntel: strMotorola); OUT;
     Tcl_Flush (outChan);
 }
 #undef OUT
@@ -230,7 +231,7 @@ static Boln readUShortRow (Tcl_Interp *interp, tkimg_MFile *handle, UShort *pixe
     if (2 * nShorts != tkimg_Read2(handle, buf, 2 * nShorts)) {
         return FALSE;
     }
-             
+
     if (swapBytes) {
         for (i=0; i<nShorts; i++) {
             ((char *)mPtr)[0] = bufPtr[1];
@@ -256,7 +257,7 @@ static Boln readUByteRow (Tcl_Interp *interp, tkimg_MFile *handle, UByte *pixels
     char  *bufPtr = buf;
     UInt  i, val;
 
-    #ifdef DEBUG_LOCAL 
+    #ifdef DEBUG_LOCAL
         printf ("Reading %d UBytes\n", nBytes);
     #endif
     if (isAscii) {
@@ -272,7 +273,7 @@ static Boln readUByteRow (Tcl_Interp *interp, tkimg_MFile *handle, UByte *pixels
     if (nBytes != tkimg_Read2(handle, buf, nBytes)) {
         return FALSE;
     }
-             
+
     for (i=0; i<nBytes; i++) {
         ((char *)mPtr)[0] = bufPtr[0];
         mPtr++;
@@ -282,22 +283,26 @@ static Boln readUByteRow (Tcl_Interp *interp, tkimg_MFile *handle, UByte *pixels
 }
 
 static Boln readUShortFile (Tcl_Interp *interp, tkimg_MFile *handle, UShort *buf, Int width, Int height,
-                            Int nchan, Boln swapBytes, Boln isAscii, Boln verbose, 
-                            Float minVals[], Float maxVals[])
+                            Int nchan, Boln swapBytes, Boln isAscii, Boln verbose,
+                            Double minVals[], Double maxVals[])
 {
     Int    x, y, c;
     UShort *bufPtr = buf;
     char   *line;
 
-    #ifdef DEBUG_LOCAL 
+    #ifdef DEBUG_LOCAL
         printf ("readUShortFile: Width=%d Height=%d nchan=%d swapBytes=%s\n",
                  width, height, nchan, swapBytes? "yes": "no");
     #endif
     for (c=0; c<nchan; c++) {
-        minVals[c] =  (Float)1.0E30;
-        maxVals[c] = (Float)-1.0E30;
+        minVals[c] =  DBL_MAX;
+        maxVals[c] = -DBL_MAX;
+    }
+    line = attemptckalloc (sizeof (UShort) * nchan * width);
+    if (line == NULL) {
+        Tcl_AppendResult (interp, "Unable to allocate memory for image data.", (char *) NULL);
+        return FALSE;
     }
-    line = ckalloc (sizeof (UShort) * nchan * width);
 
     for (y=0; y<height; y++) {
         if (!readUShortRow (interp, handle, bufPtr, nchan * width, line, swapBytes, isAscii)) {
@@ -314,12 +319,12 @@ static Boln readUShortFile (Tcl_Interp *interp, tkimg_MFile *handle, UShort *buf
     if (verbose) {
         printf ("\tMinimum pixel values :");
         for (c=0; c<nchan; c++) {
-            printf (" %d", (UShort)minVals[c]);
+            printf (" %u", (UShort)minVals[c]);
         }
         printf ("\n");
         printf ("\tMaximum pixel values :");
         for (c=0; c<nchan; c++) {
-            printf (" %d", (UShort)maxVals[c]);
+            printf (" %u", (UShort)maxVals[c]);
         }
         printf ("\n");
         fflush (stdout);
@@ -329,22 +334,26 @@ static Boln readUShortFile (Tcl_Interp *interp, tkimg_MFile *handle, UShort *buf
 }
 
 static Boln readUByteFile (Tcl_Interp *interp, tkimg_MFile *handle, UByte *buf, Int width, Int height,
-                           Int nchan, Boln swapBytes, Boln isAscii, Boln verbose, 
-                           Float minVals[], Float maxVals[])
+                           Int nchan, Boln swapBytes, Boln isAscii, Boln verbose,
+                           Double minVals[], Double maxVals[])
 {
     Int   x, y, c;
     UByte *bufPtr = buf;
     char  *line;
 
-    #ifdef DEBUG_LOCAL 
+    #ifdef DEBUG_LOCAL
         printf ("readUByteFile: Width=%d Height=%d nchan=%d swapBytes=%s\n",
                  width, height, nchan, swapBytes? "yes": "no");
     #endif
     for (c=0; c<nchan; c++) {
-        minVals[c] =  (Float)1.0E30;
-        maxVals[c] = (Float)-1.0E30;
+        minVals[c] =  DBL_MAX;
+        maxVals[c] = -DBL_MAX;
+    }
+    line = attemptckalloc (sizeof (UByte) * nchan * width);
+    if (line == NULL) {
+        Tcl_AppendResult (interp, "Unable to allocate memory for image data.", (char *) NULL);
+        return FALSE;
     }
-    line = ckalloc (sizeof (UByte) * nchan * width);
 
     for (y=0; y<height; y++) {
         if (!readUByteRow (interp, handle, bufPtr, nchan * width, line, swapBytes, isAscii)) {
@@ -361,12 +370,12 @@ static Boln readUByteFile (Tcl_Interp *interp, tkimg_MFile *handle, UByte *buf,
     if (verbose) {
         printf ("\tMinimum pixel values :");
         for (c=0; c<nchan; c++) {
-            printf (" %d", (UByte)minVals[c]);
+            printf (" %u", (UByte)minVals[c]);
         }
         printf ("\n");
         printf ("\tMaximum pixel values :");
         for (c=0; c<nchan; c++) {
-            printf (" %d", (UByte)maxVals[c]);
+            printf (" %u", (UByte)maxVals[c]);
         }
         printf ("\n");
         fflush (stdout);
@@ -402,7 +411,7 @@ static int ParseFormatOpts(
     }
     if (objc) {
         for (i=1; i<objc; i++) {
-            if (Tcl_GetIndexFromObj(interp, objv[i], (const char *CONST86 *)ppmOptions,
+            if (Tcl_GetIndexFromObj(interp, objv[i], (const char * const *)ppmOptions,
                     "format option", 0, &index) != TCL_OK) {
                 return TCL_ERROR;
             }
@@ -421,7 +430,7 @@ static int ParseFormatOpts(
                                           (char *) NULL);
                         return TCL_ERROR;
                     }
-                    opts->verbose = boolVal; 
+                    opts->verbose = boolVal;
                     break;
                 case 1:
                     if (Tcl_GetDouble(interp, optionStr, &doubleVal) == TCL_ERROR) {
@@ -493,7 +502,7 @@ static int CommonRead (Tcl_Interp *interp, tkimg_MFile *handle,
 static int CommonWrite (Tcl_Interp *interp,
                 const char *filename, Tcl_Obj *format,
                 tkimg_MFile *handle, Tk_PhotoImageBlock *blockPtr);
-static int ReadPPMFileHeader (tkimg_MFile *handle, int *widthPtr, 
+static int ReadPPMFileHeader (tkimg_MFile *handle, int *widthPtr,
                 int *heightPtr, int *maxIntensityPtr, Boln *isAsciiPtr);
 
 
@@ -638,11 +647,11 @@ static int CommonRead(
     Boln swapBytes, isAscii;
     int stopY, outY;
     int bytesPerPixel;
-    Float minVals[IMG_MAX_CHANNELS], maxVals[IMG_MAX_CHANNELS];
+    Double minVals[IMG_MAX_CHANNELS], maxVals[IMG_MAX_CHANNELS];
     UByte  *pixbufPtr;
     UShort *ushortBufPtr;
     UByte  *ubyteBufPtr;
-    Float  gtable[IMG_GAMMA_TABLE_SIZE];
+    Double gtable[IMG_GAMMA_TABLE_SIZE];
 
     memset (&tf, 0, sizeof (PPMFILE));
 
@@ -655,7 +664,7 @@ static int CommonRead(
     type = ReadPPMFileHeader (handle, &fileWidth, &fileHeight, &maxIntensity, &isAscii);
     if (type == 0) {
         Tcl_AppendResult(interp, "couldn't read PPM header from file \"",
-                          filename, "\"", NULL);
+                          filename, "\"", (char *)NULL);
         return TCL_ERROR;
     }
 
@@ -667,7 +676,7 @@ static int CommonRead(
     if ((maxIntensity <= 0) || (maxIntensity >= 65536)) {
         char buffer[TCL_INTEGER_SPACE];
 
-        sprintf(buffer, "%d", maxIntensity);
+        tkimg_snprintf(buffer, TCL_INTEGER_SPACE, "%d", maxIntensity);
         Tcl_AppendResult(interp, "PPM image file \"", filename,
                           "\" has bad maximum intensity value ", buffer,
                           (char *) NULL);
@@ -691,7 +700,8 @@ static int CommonRead(
     }
     if ((width <= 0) || (height <= 0)
         || (srcX >= fileWidth) || (srcY >= fileHeight)) {
-        return TCL_OK;
+        Tcl_AppendResult(interp, "Width or height are negative", (char *) NULL);
+        return TCL_ERROR;
     }
 
     if (type == PGM) {
@@ -707,12 +717,21 @@ static int CommonRead(
     block.width = width;
     block.height = 1;
     block.pitch = block.pixelSize * fileWidth;
-    tf.pixbuf = (UByte *) ckalloc (fileWidth * block.pixelSize);
+    tf.pixbuf = (UByte *) attemptckalloc (fileWidth * block.pixelSize);
+    if (tf.pixbuf == NULL) {
+        Tcl_AppendResult (interp, "Unable to allocate memory for image data.", (char *) NULL);
+        return TCL_ERROR;
+    }
     block.pixelPtr = tf.pixbuf + srcX * block.pixelSize;
 
     switch (bytesPerPixel) {
         case 2: {
-            tf.ushortBuf = (UShort *)ckalloc (fileWidth*fileHeight*block.pixelSize*sizeof (UShort));
+            tf.ushortBuf = (UShort *)attemptckalloc (fileWidth*fileHeight*block.pixelSize*sizeof (UShort));
+            if (tf.ushortBuf == NULL) {
+                Tcl_AppendResult (interp, "Unable to allocate memory for image data.", (char *) NULL);
+                ppmClose (&tf);
+                return TCL_ERROR;
+            }
             if (!readUShortFile(interp, handle, tf.ushortBuf, fileWidth, fileHeight, block.pixelSize,
                                  swapBytes, isAscii, opts.verbose, minVals, maxVals)) {
                 ppmClose (&tf);
@@ -721,7 +740,12 @@ static int CommonRead(
             break;
         }
         case 1: {
-            tf.ubyteBuf = (UByte *)ckalloc (fileWidth*fileHeight*block.pixelSize*sizeof (UByte));
+            tf.ubyteBuf = (UByte *)attemptckalloc (fileWidth*fileHeight*block.pixelSize*sizeof (UByte));
+            if (tf.ubyteBuf == NULL) {
+                Tcl_AppendResult (interp, "Unable to allocate memory for image data.", (char *) NULL);
+                ppmClose (&tf);
+                return TCL_ERROR;
+            }
             if (!readUByteFile (interp, handle, tf.ubyteBuf, fileWidth, fileHeight, block.pixelSize,
                         swapBytes, isAscii, opts.verbose, minVals, maxVals)) {
                 ppmClose (&tf);
@@ -741,7 +765,7 @@ static int CommonRead(
         case 2: {
             tkimg_RemapUShortValues (
                 tf.ushortBuf, fileWidth, fileHeight,
-                block.pixelSize, minVals, maxVals
+                block.pixelSize, minVals, maxVals, -1.0f, 0
             );
             break;
         }
@@ -764,7 +788,7 @@ static int CommonRead(
                 } else {
                     ushortBufPtr = tf.ushortBuf + y * fileWidth * block.pixelSize;
                 }
-                tkimg_UShortToUByte (fileWidth * block.pixelSize, ushortBufPtr, 
+                tkimg_UShortToUByte (fileWidth * block.pixelSize, ushortBufPtr,
                                      opts.gamma != 1.0? gtable: NULL, pixbufPtr);
                 ushortBufPtr += fileWidth * block.pixelSize;
                 break;
@@ -784,7 +808,7 @@ static int CommonRead(
         }
         if (y >= srcY) {
             if (tkimg_PhotoPutBlock(interp, imageHandle, &block, destX, outY,
-                                width, 1, 
+                                width, 1,
                                 block.offset[3]?
                                 TK_PHOTO_COMPOSITE_SET:
                                 TK_PHOTO_COMPOSITE_OVERLAY) == TCL_ERROR) {
@@ -870,7 +894,7 @@ static int writeAsciiRow (tkimg_MFile *handle, const unsigned char *scanline, in
     char buf[TCL_INTEGER_SPACE];
 
     for (i=0; i<nBytes; i++) {
-        sprintf (buf, "%d\n", scanline[i]);
+        tkimg_snprintf (buf, TCL_INTEGER_SPACE, "%d\n", scanline[i]);
         if (tkimg_Write2(handle, buf, strlen(buf)) != strlen(buf)) {
             return i;
         }
@@ -896,8 +920,9 @@ static int CommonWrite(
         return TCL_ERROR;
     }
 
-    sprintf(header, "P%d\n%d %d\n255\n", opts.writeAscii? 3: 6,
-                     blockPtr->width, blockPtr->height);
+    tkimg_snprintf(header, 16 + TCL_INTEGER_SPACE * 2,
+                  "P%d\n%d %d\n255\n", opts.writeAscii? 3: 6,
+                   blockPtr->width, blockPtr->height);
     if (tkimg_Write2(handle, header, strlen(header)) != strlen(header)) {
         goto writeerror;
     }
@@ -908,7 +933,11 @@ static int CommonWrite(
     blueOff    = blockPtr->offset[2] - blockPtr->offset[0];
 
     nBytes = blockPtr->width * 3; /* Only RGB images allowed. */
-    scanline = (unsigned char *) ckalloc((unsigned) nBytes);
+    scanline = (unsigned char *) attemptckalloc((unsigned) nBytes);
+    if (scanline == NULL) {
+        Tcl_AppendResult (interp, "Unable to allocate memory for image data.", (char *) NULL);
+        goto writeerror;
+    }
     for (h = blockPtr->height; h > 0; h--) {
         pixelPtr = pixLinePtr;
         scanlinePtr = scanline;
