@@ -14,10 +14,14 @@
  *
  * The following image types are currently supported:
  *
- * Grayscale image:  1 channel  of 32-bit floating point   values.
+ * Grayscale image:  1 channel  of 64-bit floating point   values.
+ *                   1 channel  of 32-bit floating point   values.
+ *                   1 channel  of 32-bit unsigned integer values.
  *                   1 channel  of 16-bit unsigned integer values.
  *                   1 channel  of  8-bit unsigned integer values.
- * True-color image: 3 channels of 32-bit floating point   values.
+ * True-color image: 3 channels of 64-bit floating point   values.
+ *                   3 channels of 32-bit floating point   values.
+ *                   3 channels of 32-bit unsigned integer values.
  *                   3 channels of 16-bit unsigned integer values.
  *                   3 channels of  8-bit unsigned integer values.
  *
@@ -39,7 +43,7 @@
  *     NumChan=1\n              Possible values: 1 or 3.
  *     ByteOrder=Intel\n        Possible values: "Intel" or "Motorola".
  *     ScanOrder=TopDown\n      Possible values: "TopDown" or "BottomUp".
- *     PixelType=byte\n         Possible values: "float", "short" or "byte".
+ *     PixelType=byte\n         Possible values: "double", "float", "int", "short", "byte".
  *
  * The following format options are available:
  *
@@ -55,7 +59,7 @@
  *                       -gamma <float> -min <float> -max <float>
  *                       -cutoff <float> -saturation <float> -printagc <bool>
  *                       -nchan <int> -scanorder <string> -byteorder <string>
- *                       -width <int> -height <int> 
+ *                       -width <int> -height <int> -skipbytes <int>
  *                       -pixeltype <string> -uuencode <bool>"
  * Write RAW image: "raw -useheader false -verbose <bool> -nchan <int>
  *                       -scanorder <string>"
@@ -72,7 +76,7 @@
  *
  *                      Mode "none":
  *                      If mapping mode is set to "none", no mapping of input
- *                      values is done. Use this mode, if the image already 
+ *                      values is done. Use this mode, if the image already
  *                      contains RGB values in the range of 0 ..255.
  *                      When using mode "none", no information about the
  *                      minimum and maximum pixel values is gathered during
@@ -85,7 +89,7 @@
  *
  *                      Mode "agc":
  *                      "agc" applies an automatic gain control algorithmn to the
- *                      image data. 
+ *                      image data.
  *                      Currently implemented for 1-channel 32-bit float images only.
  * -gamma <float>:      Specify a gamma correction to be applied when mapping
  *                      the input data to 8-bit image values.
@@ -99,7 +103,7 @@
  *                      Default is the minimum value found in the image data.
  * -saturation <float>: If option is given, an Automatic Gain Control algorithmn is
  *                      applied to the input values. The supplied value specifies the
- *                      saturation value, i.e. all pixel values greater than the 
+ *                      saturation value, i.e. all pixel values greater than the
  *                      saturation are mapped to white.
  *                      Valid for mapping mode: agc
  * -cutoff <float>:     If option is given, an Automatic Gain Control algorithmn is
@@ -117,6 +121,8 @@
  *                      reading image data without header. Default is 128.
  * -height <int>:       Specify the height of the input image. Only valid, if
  *                      reading image data without header. Default is 128.
+ * -skipbytes <int>:    Specify the number of bytes to skip before reading image data.
+ *                      Only valid, if reading image data without header. Default is 0.
  * -byteorder <string>: Specify the byteorder of the input image. Only valid, if
  *                      reading image data without header.
  *                      Possible values: "Intel" or "Motorola".
@@ -128,7 +134,7 @@
  *                      Default is "TopDown".
  * -pixeltype <string>: Specify the type of the pixel values.
  *                      Only valid, if reading image data without header.
- *                      Possible values: "float", "short" or "byte".
+ *                      Possible values: "double", "float", "int", "short", "byte".
  *                      Default is "byte".
  * -uuencode <bool>:    If set to false, do not assume, that the image data stored in a
  *                      variable is uuencoded. Default is true, i.e. the image data is
@@ -167,7 +173,9 @@
 #define strMotorola "Motorola"
 #define strTopDown  "TopDown"
 #define strBottomUp "BottomUp"
+#define strDouble   "double"
 #define strFloat    "float"
+#define strUInt     "int"
 #define strUShort   "short"
 #define strUByte    "byte"
 
@@ -177,9 +185,11 @@
 #define TOP_DOWN    1
 #define INTEL       0
 #define MOTOROLA    1
-#define TYPE_FLOAT  0
-#define TYPE_USHORT 1
-#define TYPE_UBYTE  2
+#define TYPE_DOUBLE 0
+#define TYPE_FLOAT  1
+#define TYPE_UINT   2
+#define TYPE_USHORT 3
+#define TYPE_UBYTE  4
 
 /* Some general defines and typedefs. */
 #define TRUE  1
@@ -190,7 +200,7 @@ typedef unsigned char UByte;    /* Unsigned  8 bit integer */
 typedef char  Byte;             /* Signed    8 bit integer */
 typedef unsigned short UShort;  /* Unsigned 16 bit integer */
 typedef short Short;            /* Signed   16 bit integer */
-typedef int UInt;               /* Unsigned 32 bit integer */
+typedef unsigned int UInt;      /* Unsigned 32 bit integer */
 typedef int Int;                /* Signed   32 bit integer */
 typedef float Float;            /* IEEE     32 bit floating point */
 typedef double Double;          /* IEEE     64 bit floating point */
@@ -208,29 +218,32 @@ typedef struct {
 
 /* Format options structure for use with ParseFormatOpts */
 typedef struct {
-    Int   width;
-    Int   height;
-    Int   nchan;
-    Int   scanOrder;
-    Int   byteOrder;
-    Int   pixelType;
-    Int   mapMode;
-    Float gamma;        /* IMG_MAP_MINMAX and IMG_MAP_AGC */
-    Float minVal;       /* IMG_MAP_MINMAX */
-    Float maxVal;       /* IMG_MAP_MINMAX */
-    Float saturation;   /* IMG_MAP_AGC */
-    Float cutOff;       /* IMG_MAP_AGC */
-    Boln  verbose;
-    Boln  printAgc;
-    Boln  uuencode;
-    Boln  useHeader;
+    Int    width;
+    Int    height;
+    Int    skipBytes;
+    Int    nchan;
+    Int    scanOrder;
+    Int    byteOrder;
+    Int    pixelType;
+    Int    mapMode;
+    Double gamma;        /* IMG_MAP_MINMAX and IMG_MAP_AGC */
+    Double minVal;       /* IMG_MAP_MINMAX */
+    Double maxVal;       /* IMG_MAP_MINMAX */
+    Double saturation;   /* IMG_MAP_AGC */
+    Double cutOff;       /* IMG_MAP_AGC */
+    Boln   verbose;
+    Boln   printAgc;
+    Boln   uuencode;
+    Boln   useHeader;
 } FMTOPT;
 
 /* Structure to hold information about the image file being processed. */
 typedef struct {
     RAWHEADER th;
     UByte  *pixbuf;
+    Double *doubleBuf;
     Float  *floatBuf;
+    UInt   *uintBuf;
     UShort *ushortBuf;
     UByte  *ubyteBuf;
 } RAWFILE;
@@ -240,7 +253,9 @@ static void rawClose (RAWFILE *tf, Boln fastMode)
     if (!fastMode) {
         if (tf->pixbuf) ckfree ((char *)tf->pixbuf);
     }
+    if (tf->doubleBuf) ckfree ((char *)tf->doubleBuf);
     if (tf->floatBuf)  ckfree ((char *)tf->floatBuf);
+    if (tf->uintBuf)   ckfree ((char *)tf->uintBuf);
     if (tf->ushortBuf) ckfree ((char *)tf->ushortBuf);
     if (tf->ubyteBuf)  ckfree ((char *)tf->ubyteBuf);
     return;
@@ -257,31 +272,33 @@ static void printImgInfo (RAWHEADER *th, FMTOPT *opts,
     if (!outChan) {
         return;
     }
-    sprintf (str, "%s %s\n", msg, filename);                                                       OUT;
-    sprintf (str, "\tSize in pixel    : %d x %d\n", th->width, th->height);                        OUT;
-    sprintf (str, "\tNo. of channels  : %d\n",      th->nChans);                                   OUT;
-    sprintf (str, "\tPixel type       : %s\n",      (th->pixelType == TYPE_FLOAT?  strFloat:
-                                                    (th->pixelType == TYPE_USHORT? strUShort:
-                                                    (th->pixelType == TYPE_UBYTE?  strUByte:
-                                                                                   strUnknown)))); OUT;
-    sprintf (str, "\tVertical encoding: %s\n",      th->scanOrder == TOP_DOWN?
-                                                    strTopDown: strBottomUp);                      OUT;
-    sprintf (str, "\tHost byte order  : %s\n",      tkimg_IsIntel ()?  strIntel: strMotorola);     OUT;
-    sprintf (str, "\tFile byte order  : %s\n",      th->byteOrder == INTEL?
-                                                    strIntel: strMotorola);                        OUT;
-    sprintf (str, "\tMapping mode     : %s\n",      (opts->mapMode == IMG_MAP_NONE?   IMG_MAP_NONE_STR:
-                                                    (opts->mapMode == IMG_MAP_MINMAX? IMG_MAP_MINMAX_STR:
-                                                    (opts->mapMode == IMG_MAP_AGC?    IMG_MAP_AGC_STR:
-                                                                                  strUnknown))));  OUT;
+    tkimg_snprintf (str, 256, "%s %s\n", msg, filename);                                                         OUT;
+    tkimg_snprintf (str, 256, "\tSize in pixel    : %d x %d\n", th->width, th->height);                          OUT;
+    tkimg_snprintf (str, 256, "\tNo. of channels  : %d\n",      th->nChans);                                     OUT;
+    tkimg_snprintf (str, 256, "\tPixel type       : %s\n",      (th->pixelType == TYPE_DOUBLE? strDouble:
+                                                                (th->pixelType == TYPE_FLOAT?  strFloat:
+                                                                (th->pixelType == TYPE_UINT?   strUInt:
+                                                                (th->pixelType == TYPE_USHORT? strUShort:
+                                                                (th->pixelType == TYPE_UBYTE?  strUByte:
+                                                                                               strUnknown)))))); OUT;
+    tkimg_snprintf (str, 256, "\tVertical encoding: %s\n",      th->scanOrder == TOP_DOWN?
+                                                                strTopDown: strBottomUp);                        OUT;
+    tkimg_snprintf (str, 256, "\tHost byte order  : %s\n",      tkimg_IsIntel ()?  strIntel: strMotorola);       OUT;
+    tkimg_snprintf (str, 256, "\tFile byte order  : %s\n",      th->byteOrder == INTEL?
+                                                                strIntel: strMotorola);                          OUT;
+    tkimg_snprintf (str, 256, "\tMapping mode     : %s\n",      (opts->mapMode == IMG_MAP_NONE?   IMG_MAP_NONE_STR:
+                                                                (opts->mapMode == IMG_MAP_MINMAX? IMG_MAP_MINMAX_STR:
+                                                                (opts->mapMode == IMG_MAP_AGC?    IMG_MAP_AGC_STR:
+                                                                                              strUnknown))));    OUT;
     if (opts->mapMode != IMG_MAP_NONE) {
-        sprintf (str, "\tGamma correction : %f\n",       opts->gamma);                             OUT;
+        tkimg_snprintf (str, 256, "\tGamma correction : %lf\n",       opts->gamma);                              OUT;
         if (opts->mapMode == IMG_MAP_MINMAX) {
-            sprintf (str, "\tMinimum map value: %f\n",   opts->minVal);                            OUT;
-            sprintf (str, "\tMaximum map value: %f\n",   opts->maxVal);                            OUT;
+            tkimg_snprintf (str, 256, "\tMinimum map value: %lf\n",   opts->minVal);                             OUT;
+            tkimg_snprintf (str, 256, "\tMaximum map value: %lf\n",   opts->maxVal);                             OUT;
         }
         if (opts->mapMode == IMG_MAP_AGC) {
-            sprintf (str, "\tSaturation       : %f\n",   opts->saturation);                        OUT;
-            sprintf (str, "\tCutOff           : %f%%\n", opts->cutOff);                            OUT;
+            tkimg_snprintf (str, 256, "\tSaturation       : %lf\n",   opts->saturation);                         OUT;
+            tkimg_snprintf (str, 256, "\tCutOff           : %lf%%\n", opts->cutOff);                             OUT;
         }
     }
     Tcl_Flush (outChan);
@@ -293,6 +310,7 @@ static Boln readHeaderLine (Tcl_Interp *interp, tkimg_MFile *handle, char *buf)
     char c, *bufPtr, *bufEndPtr;
     Boln failure;
 
+    buf[0]    = '\0';
     bufPtr    = buf;
     bufEndPtr = buf + HEADLEN;
     failure   = TRUE;
@@ -307,8 +325,7 @@ static Boln readHeaderLine (Tcl_Interp *interp, tkimg_MFile *handle, char *buf)
         bufPtr++;
     }
     if (failure) {
-        Tcl_AppendResult (interp, "RAW handler: Error reading header line (",
-                          buf, ")\n", NULL);
+        Tcl_AppendResult (interp, "RAW handler: Error reading header line\n", (char *)NULL);
         return FALSE;
     }
     return TRUE;
@@ -321,51 +338,51 @@ static Boln readHeader (Tcl_Interp *interp, tkimg_MFile *handle, RAWHEADER *th)
 
     if (!readHeaderLine (interp, handle, buf) ||
         (1 != sscanf (buf, strMagic, th->id))) {
-        Tcl_AppendResult (interp, "Unable to parse header field Magic\n", NULL);
+        Tcl_AppendResult (interp, "Unable to parse header field Magic\n", (char *)NULL);
         return FALSE;
     }
     if (strcmp (th->id, "RAW") != 0) {
         Tcl_AppendResult (interp, "Invalid value for header field Magic:",
-                                  "Must be \"RAW\"\n", NULL);
+                                  "Must be \"RAW\"\n", (char *)NULL);
         return FALSE;
     }
 
     if (!readHeaderLine (interp, handle, buf) ||
         (1 != sscanf (buf, strWidth, &th->width))) {
-        Tcl_AppendResult (interp, "Unable to parse header field Width\n", NULL);
+        Tcl_AppendResult (interp, "Unable to parse header field Width\n", (char *)NULL);
         return FALSE;
     }
     if (th->width < 1) {
         Tcl_AppendResult (interp, "Invalid value for header field Width:",
-                                  "Must be greater than zero\n", NULL);
+                                  "Must be greater than zero\n", (char *)NULL);
         return FALSE;
     }
 
     if (!readHeaderLine (interp, handle, buf) ||
         (1 != sscanf (buf, strHeight, &th->height))) {
-        Tcl_AppendResult (interp, "Unable to parse header field Height\n", NULL);
+        Tcl_AppendResult (interp, "Unable to parse header field Height\n", (char *)NULL);
         return FALSE;
     }
     if (th->height < 1) {
         Tcl_AppendResult (interp, "Invalid value for header field Height:",
-                                  "Must be greater than zero\n", NULL);
+                                  "Must be greater than zero\n", (char *)NULL);
         return FALSE;
     }
 
     if (!readHeaderLine (interp, handle, buf) ||
         (1 != sscanf (buf, strNumChan, &th->nChans))) {
-        Tcl_AppendResult (interp, "Unable to parse header field NumChan\n", NULL);
+        Tcl_AppendResult (interp, "Unable to parse header field NumChan\n", (char *)NULL);
         return FALSE;
     }
     if (th->nChans != 1 && th->nChans != 3) {
         Tcl_AppendResult (interp, "Invalid value for header field NumChan:",
-                                  "Must be 1 or 3\n", NULL);
+                                  "Must be 1 or 3\n", (char *)NULL);
         return FALSE;
     }
 
     if (!readHeaderLine (interp, handle, buf) ||
         (1 != sscanf (buf, strByteOrder, tmpStr))) {
-        Tcl_AppendResult (interp, "Unable to parse header field ByteOrder\n", NULL);
+        Tcl_AppendResult (interp, "Unable to parse header field ByteOrder\n", (char *)NULL);
         return FALSE;
     }
 
@@ -376,13 +393,13 @@ static Boln readHeader (Tcl_Interp *interp, tkimg_MFile *handle, RAWHEADER *th)
     } else {
         Tcl_AppendResult (interp, "Invalid value for header field ByteOrder:",
                                   "Must be ", strIntel, " or ", strMotorola,
-                                  "\n", NULL);
+                                  "\n", (char *)NULL);
         return FALSE;
     }
 
     if (!readHeaderLine (interp, handle, buf) ||
         (1 != sscanf (buf, strScanOrder, tmpStr))) {
-        Tcl_AppendResult (interp, "Unable to parse header field ScanOrder\n", NULL);
+        Tcl_AppendResult (interp, "Unable to parse header field ScanOrder\n", (char *)NULL);
         return FALSE;
     }
     if (strcmp (tmpStr, strTopDown) == 0) {
@@ -392,25 +409,29 @@ static Boln readHeader (Tcl_Interp *interp, tkimg_MFile *handle, RAWHEADER *th)
     } else {
         Tcl_AppendResult (interp, "Invalid value for header field ScanOrder:",
                                   "Must be ", strTopDown, " or ", strBottomUp,
-                                  "\n", NULL);
+                                  "\n", (char *)NULL);
         return FALSE;
     }
 
     if (!readHeaderLine (interp, handle, buf) ||
         (1 != sscanf (buf, strPixelType, tmpStr))) {
-        Tcl_AppendResult (interp, "Unable to parse header field PixelType\n", NULL);
+        Tcl_AppendResult (interp, "Unable to parse header field PixelType\n", (char *)NULL);
         return FALSE;
     }
-    if (strcmp (tmpStr, strFloat) == 0) {
+    if (strcmp (tmpStr, strDouble) == 0) {
+        th->pixelType = TYPE_DOUBLE;
+    } else if (strcmp (tmpStr, strFloat) == 0) {
         th->pixelType = TYPE_FLOAT;
+    } else if (strcmp (tmpStr, strUInt) == 0) {
+        th->pixelType = TYPE_UINT;
     } else if (strcmp (tmpStr, strUShort) == 0) {
         th->pixelType = TYPE_USHORT;
     } else if (strcmp (tmpStr, strUByte) == 0) {
         th->pixelType = TYPE_UBYTE;
     } else {
         Tcl_AppendResult (interp, "Invalid value for header field PixelType:",
-                                  "Must be ", strFloat, ",", strUShort, " or ", strUByte,
-                                  "\n", NULL);
+                                  "Must be ", strDouble, ", ", strFloat, ", ", strUInt, ", " strUShort, " or ", strUByte,
+                                  "\n", (char *)NULL);
         return FALSE;
     }
 
@@ -421,23 +442,25 @@ static Boln writeHeader (tkimg_MFile *handle, RAWHEADER *th)
 {
     char buf[1024];
 
-    sprintf (buf, strMagic, "RAW");
+    tkimg_snprintf (buf, 1024, strMagic, "RAW");
     tkimg_Write2(handle, buf, strlen (buf));
-    sprintf (buf, strWidth, th->width);
+    tkimg_snprintf (buf, 1024, strWidth, th->width);
     tkimg_Write2(handle, buf, strlen (buf));
-    sprintf (buf, strHeight, th->height);
+    tkimg_snprintf (buf, 1024, strHeight, th->height);
     tkimg_Write2(handle, buf, strlen (buf));
-    sprintf (buf, strNumChan, th->nChans);
+    tkimg_snprintf (buf, 1024, strNumChan, th->nChans);
     tkimg_Write2(handle, buf, strlen (buf));
-    sprintf (buf, strByteOrder, tkimg_IsIntel()? strIntel: strMotorola);
+    tkimg_snprintf (buf, 1024, strByteOrder, tkimg_IsIntel()? strIntel: strMotorola);
     tkimg_Write2(handle, buf, strlen (buf));
-    sprintf (buf, strScanOrder, th->scanOrder == TOP_DOWN?
-                                strTopDown: strBottomUp);
+    tkimg_snprintf (buf, 1024, strScanOrder, th->scanOrder == TOP_DOWN?
+                                             strTopDown: strBottomUp);
     tkimg_Write2(handle, buf, strlen (buf));
-    sprintf (buf, strPixelType, (th->pixelType == TYPE_FLOAT?  strFloat:
-                                (th->pixelType == TYPE_USHORT? strUShort:
-                                (th->pixelType == TYPE_UBYTE?  strUByte:
-                                                               strUnknown))));
+    tkimg_snprintf (buf, 1024, strPixelType, (th->pixelType == TYPE_DOUBLE? strDouble:
+                                             (th->pixelType == TYPE_FLOAT?  strFloat:
+                                             (th->pixelType == TYPE_UINT?   strUInt:
+                                             (th->pixelType == TYPE_USHORT? strUShort:
+                                             (th->pixelType == TYPE_UBYTE?  strUByte:
+                                                                            strUnknown))))));
     tkimg_Write2(handle, buf, strlen (buf));
     return TRUE;
 }
@@ -485,8 +508,8 @@ static int ParseFormatOpts(
     static const char *const rawOptions[] = {
          "-verbose", "-width", "-height", "-nchan", "-byteorder",
          "-scanorder", "-pixeltype", "-min", "-max", "-gamma",
-         "-useheader", "-map", "-uuencode", "-saturation", "-cutoff", 
-         "-nomap", "-printagc", NULL
+         "-useheader", "-map", "-uuencode", "-saturation", "-cutoff",
+         "-nomap", "-printagc", "-skipbytes", NULL
     };
     int objc, i, index;
     char *optionStr;
@@ -512,12 +535,13 @@ static int ParseFormatOpts(
     opts->saturation = -1.0;
     opts->cutOff     = 3.0;
     opts->printAgc   = 0;
+    opts->skipBytes  = 0;
 
     if (tkimg_ListObjGetElements (interp, format, &objc, &objv) != TCL_OK)
         return TCL_ERROR;
     if (objc) {
         for (i=1; i<objc; i++) {
-            if (Tcl_GetIndexFromObj (interp, objv[i], (const char *CONST86 *)rawOptions,
+            if (Tcl_GetIndexFromObj (interp, objv[i], (const char * const *)rawOptions,
                     "format option", 0, &index) != TCL_OK) {
                 return TCL_ERROR;
             }
@@ -589,15 +613,19 @@ static int ParseFormatOpts(
                     }
                     break;
                 case 6:
-                    if (!strncmp (optionStr, strFloat, strlen (strFloat))) {
+                    if (!strncmp (optionStr, strDouble, strlen (strDouble))) {
+                        opts->pixelType = TYPE_DOUBLE;
+                    } else if (!strncmp (optionStr, strFloat, strlen (strFloat))) {
                         opts->pixelType = TYPE_FLOAT;
+                    } else if (!strncmp (optionStr, strUInt, strlen (strUInt))) {
+                        opts->pixelType = TYPE_UINT;
                     } else if (!strncmp (optionStr, strUShort, strlen (strUShort))) {
                         opts->pixelType = TYPE_USHORT;
                     } else if (!strncmp (optionStr, strUByte, strlen (strUByte))) {
                         opts->pixelType = TYPE_UBYTE;
                     } else {
                         Tcl_AppendResult (interp, "Invalid pixel type \"", optionStr,
-                                          "\": should be float, short or byte",
+                                          "\": should be double, float, int, short or byte",
                                           (char *) NULL);
                         return TCL_ERROR;
                     }
@@ -705,6 +733,14 @@ static int ParseFormatOpts(
                     }
                     opts->printAgc = boolVal;
                     break;
+                case 17:
+                    if (Tcl_GetInt(interp, optionStr, &intVal) == TCL_ERROR || intVal < 0) {
+                        Tcl_AppendResult (interp, "Invalid byte skip value \"", optionStr,
+                                          "\": Must be equal to or greater than zero.", (char *) NULL);
+                        return TCL_ERROR;
+                    }
+                    opts->skipBytes = intVal;
+                    break;
             }
         }
     }
@@ -767,6 +803,7 @@ static int CommonMatch(
 ) {
     RAWHEADER th;
     FMTOPT opts;
+    char * buf;
 
     initHeader (&th);
 
@@ -784,6 +821,17 @@ static int CommonMatch(
         th.pixelType = opts.pixelType;
         th.scanOrder = opts.scanOrder;
         th.byteOrder = opts.byteOrder;
+        if (opts.skipBytes > 0) {
+            buf = (char *)attemptckalloc (opts.skipBytes);
+            if (buf == NULL) {
+                Tcl_AppendResult (interp, "Unable to allocate memory for image data.", (char *) NULL);
+                return FALSE;
+            }
+            if (opts.skipBytes != tkimg_Read2(handle, buf, opts.skipBytes)) {
+                return 0;
+            }
+            ckfree (buf);
+        }
     }
     *widthPtr  = th.width;
     *heightPtr = th.height;
@@ -859,7 +907,7 @@ static int CommonRead(
     Tk_PhotoImageBlock block;
     Int x, y, c;
     Int fileWidth = 0, fileHeight = 0;
-    Float minVals[IMG_MAX_CHANNELS], maxVals[IMG_MAX_CHANNELS];
+    Double minVals[IMG_MAX_CHANNELS], maxVals[IMG_MAX_CHANNELS];
     int stopY, outY, outWidth, outHeight;
     RAWFILE tf;
     FMTOPT opts;
@@ -870,10 +918,12 @@ static int CommonRead(
     Int  pixelType;
     Int  matte = 0;
     UByte  *pixbufPtr;
+    Double *doubleBufPtr;
     Float  *floatBufPtr;
+    UInt   *uintBufPtr;
     UShort *ushortBufPtr;
     UByte  *ubyteBufPtr;
-    Float  gtable[IMG_GAMMA_TABLE_SIZE];
+    Double gtable[IMG_GAMMA_TABLE_SIZE];
     int result = TCL_OK;
 
     memset (&tf, 0, sizeof (RAWFILE));
@@ -903,7 +953,8 @@ static int CommonRead(
     }
     if ((outWidth <= 0) || (outHeight <= 0)
         || (srcX >= fileWidth) || (srcY >= fileHeight)) {
-        return TCL_OK;
+        Tcl_AppendResult(interp, "Width or height are negative", (char *) NULL);
+        return TCL_ERROR;
     }
 
     byteOrder = opts.useHeader? tf.th.byteOrder: opts.byteOrder;
@@ -920,21 +971,56 @@ static int CommonRead(
     }
 
     switch (pixelType) {
+        case TYPE_DOUBLE: {
+            tf.doubleBuf = (Double *)attemptckalloc (fileWidth*fileHeight*tf.th.nChans*sizeof (Double));
+            if (tf.doubleBuf == NULL) {
+                Tcl_AppendResult (interp, "Unable to allocate memory for image data.", (char *) NULL);
+                return TCL_ERROR;
+            }
+            tkimg_ReadDoubleFile (handle, tf.doubleBuf, fileWidth, fileHeight, tf.th.nChans,
+                                  swapBytes, opts.verbose, opts.mapMode != IMG_MAP_NONE,
+                                  minVals, maxVals, opts.saturation);
+            break;
+        }
         case TYPE_FLOAT: {
-            tf.floatBuf = (Float *)ckalloc (fileWidth*fileHeight*tf.th.nChans*sizeof (Float));
+            tf.floatBuf = (Float *)attemptckalloc (fileWidth*fileHeight*tf.th.nChans*sizeof (Float));
+            if (tf.floatBuf == NULL) {
+                Tcl_AppendResult (interp, "Unable to allocate memory for image data.", (char *) NULL);
+                return TCL_ERROR;
+            }
             tkimg_ReadFloatFile (handle, tf.floatBuf, fileWidth, fileHeight, tf.th.nChans,
-                                 swapBytes, opts.verbose, opts.mapMode != IMG_MAP_NONE, 
+                                 swapBytes, opts.verbose, opts.mapMode != IMG_MAP_NONE,
                                  minVals, maxVals, opts.saturation);
             break;
         }
+        case TYPE_UINT: {
+            tf.uintBuf = (UInt *)attemptckalloc (fileWidth*fileHeight*tf.th.nChans*sizeof (UInt));
+            if (tf.uintBuf == NULL) {
+                Tcl_AppendResult (interp, "Unable to allocate memory for image data.", (char *) NULL);
+                return TCL_ERROR;
+            }
+            tkimg_ReadUIntFile (handle, tf.uintBuf, fileWidth, fileHeight, tf.th.nChans,
+                                swapBytes, opts.verbose, opts.mapMode != IMG_MAP_NONE,
+                                minVals, maxVals, opts.saturation);
+            break;
+        }
         case TYPE_USHORT: {
-            tf.ushortBuf = (UShort *)ckalloc (fileWidth*fileHeight*tf.th.nChans*sizeof (UShort));
+            tf.ushortBuf = (UShort *)attemptckalloc (fileWidth*fileHeight*tf.th.nChans*sizeof (UShort));
+            if (tf.ushortBuf == NULL) {
+                Tcl_AppendResult (interp, "Unable to allocate memory for image data.", (char *) NULL);
+                return TCL_ERROR;
+            }
             tkimg_ReadUShortFile (handle, tf.ushortBuf, fileWidth, fileHeight, tf.th.nChans,
-                                  swapBytes, opts.verbose, opts.mapMode != IMG_MAP_NONE, minVals, maxVals);
+                                  swapBytes, opts.verbose, opts.mapMode != IMG_MAP_NONE,
+                                  minVals, maxVals, opts.saturation);
             break;
         }
         case TYPE_UBYTE: {
-            tf.ubyteBuf = (UByte *)ckalloc (fileWidth*fileHeight*tf.th.nChans*sizeof (UByte));
+            tf.ubyteBuf = (UByte *)attemptckalloc (fileWidth*fileHeight*tf.th.nChans*sizeof (UByte));
+            if (tf.ubyteBuf == NULL) {
+                Tcl_AppendResult (interp, "Unable to allocate memory for image data.", (char *) NULL);
+                return TCL_ERROR;
+            }
             tkimg_ReadUByteFile (handle, tf.ubyteBuf, fileWidth, fileHeight, tf.th.nChans,
                                  opts.verbose, opts.mapMode != IMG_MAP_NONE, minVals, maxVals);
             break;
@@ -962,12 +1048,20 @@ static int CommonRead(
             break;
         }
         case IMG_MAP_AGC: {
-            /* Nothing to do. Saturation is considered on tkimg_ReadFloatFile. */
+            /* Nothing to do. Saturation is considered in tkimg_ReadFloatFile. */
             break;
         }
     }
 
     switch (pixelType) {
+        case TYPE_DOUBLE: {
+            tkimg_RemapDoubleValues (
+                tf.doubleBuf, fileWidth, fileHeight, tf.th.nChans,
+                minVals, maxVals, opts.mapMode == IMG_MAP_AGC? opts.cutOff: -1.0,
+                opts.printAgc
+            );
+            break;
+        }
         case TYPE_FLOAT: {
             tkimg_RemapFloatValues (
                 tf.floatBuf, fileWidth, fileHeight, tf.th.nChans,
@@ -976,10 +1070,19 @@ static int CommonRead(
             );
             break;
         }
+        case TYPE_UINT: {
+            tkimg_RemapUIntValues (
+                tf.uintBuf, fileWidth, fileHeight, tf.th.nChans,
+                minVals, maxVals, opts.mapMode == IMG_MAP_AGC? opts.cutOff: -1.0,
+                opts.printAgc
+            );
+            break;
+        }
         case TYPE_USHORT: {
             tkimg_RemapUShortValues (
                 tf.ushortBuf, fileWidth, fileHeight, tf.th.nChans,
-                minVals, maxVals
+                minVals, maxVals, opts.mapMode == IMG_MAP_AGC? opts.cutOff: -1.0,
+                opts.printAgc
             );
             break;
         }
@@ -993,7 +1096,12 @@ static int CommonRead(
     if (fastMode) {
         tf.pixbuf = tf.ubyteBuf;
     } else {
-        tf.pixbuf = (UByte *) ckalloc (fileWidth * tf.th.nChans);
+        tf.pixbuf = (UByte *) attemptckalloc (fileWidth * tf.th.nChans);
+        if (tf.pixbuf == NULL) {
+            Tcl_AppendResult (interp, "Unable to allocate memory for image data.", (char *) NULL);
+            rawClose (&tf, fastMode);
+            return TCL_ERROR;
+        }
     }
 
     block.pixelSize = tf.th.nChans;
@@ -1021,6 +1129,17 @@ static int CommonRead(
         for (y=0; y<stopY; y++) {
             pixbufPtr = tf.pixbuf;
             switch (pixelType) {
+                case TYPE_DOUBLE: {
+                    if (scanOrder == BOTTOM_UP) {
+                        doubleBufPtr = tf.doubleBuf + (fileHeight -1 - y) * fileWidth * tf.th.nChans;
+                    } else {
+                        doubleBufPtr = tf.doubleBuf + y * fileWidth * tf.th.nChans;
+                    }
+                    tkimg_DoubleToUByte (fileWidth * tf.th.nChans, doubleBufPtr,
+                                         opts.gamma != 1.0? gtable: NULL, pixbufPtr);
+                    doubleBufPtr += fileWidth * tf.th.nChans;
+                    break;
+                }
                 case TYPE_FLOAT: {
                     if (scanOrder == BOTTOM_UP) {
                         floatBufPtr = tf.floatBuf + (fileHeight -1 - y) * fileWidth * tf.th.nChans;
@@ -1032,6 +1151,17 @@ static int CommonRead(
                     floatBufPtr += fileWidth * tf.th.nChans;
                     break;
                 }
+                case TYPE_UINT: {
+                    if (scanOrder == BOTTOM_UP) {
+                        uintBufPtr = tf.uintBuf + (fileHeight -1 - y) * fileWidth * tf.th.nChans;
+                    } else {
+                        uintBufPtr = tf.uintBuf + y * fileWidth * tf.th.nChans;
+                    }
+                    tkimg_UIntToUByte (fileWidth * tf.th.nChans, uintBufPtr,
+                                       opts.gamma != 1.0? gtable: NULL, pixbufPtr);
+                    uintBufPtr += fileWidth * tf.th.nChans;
+                    break;
+                }
                 case TYPE_USHORT: {
                     if (scanOrder == BOTTOM_UP) {
                         ushortBufPtr = tf.ushortBuf + (fileHeight -1 - y) * fileWidth * tf.th.nChans;
@@ -1163,7 +1293,11 @@ static int CommonWrite(
 
     writeHeader (handle, &tf.th);
     bytesPerLine = blockPtr->width * tf.th.nChans * sizeof (UByte);
-    tf.ubyteBuf = (UByte *)ckalloc (bytesPerLine);
+    tf.ubyteBuf = (UByte *)attemptckalloc (bytesPerLine);
+    if (tf.ubyteBuf == NULL) {
+        Tcl_AppendResult (interp, "Unable to allocate memory for image data.", (char *) NULL);
+        return TCL_ERROR;
+    }
 
     rowPixPtr = blockPtr->pixelPtr + blockPtr->offset[0];
     for (y = 0; y < blockPtr->height; y++) {
