@@ -69,38 +69,18 @@
 
 #include "init.c"
 
-#ifdef WIN32
-#   define TCLSEEK_WORKAROUND
-#endif
 
-#ifdef WIN32
-#include <Windows.h>
-static int TempFileName( char name[1024])
-{
-    const char *prefix = "SGI";
-    if (GetTempPath(1024, name) != 0) {
-        if (GetTempFileName(name, prefix, 0, name) != 0) {
-            return 1;
-        }
-    }
-    name[0] = '.';
-    name[1] = '\0';
-    return GetTempFileName(name, prefix, 0, name);
-}
+#ifdef _WIN32
+#   include <windows.h>
+#   define TCLSEEK_WORKAROUND
 #else
-static int TempFileName( char name[1024])
-{
-    if (! tmpnam(name)) {
-        return 0;
-    }
-    return 1;
-}
+#   include <unistd.h>
 #endif
 
 #ifdef TCLSEEK_WORKAROUND
     static int ioMode = 0; /* Needed for Windows patch */
 
-    static int MyWrite (Tcl_Channel chan, char *buf, int size)
+    static int MyWrite(Tcl_Channel chan, char *buf, int size)
     {
         if (1 == fwrite(buf, size, 1, (FILE *)chan)) {
             return size;
@@ -109,7 +89,7 @@ static int TempFileName( char name[1024])
         }
     }
 
-    static int MyClose (Tcl_Interp *interp, Tcl_Channel chan)
+    static int MyClose(Tcl_Interp *interp, Tcl_Channel chan)
     {
         if (0 == fclose((FILE *)chan)) {
             return TCL_OK;
@@ -118,10 +98,10 @@ static int TempFileName( char name[1024])
         }
     }
 
-    static int MySeek (Tcl_Channel chan, int offset, int seekMode)
+    static int MySeek(Tcl_Channel chan, int offset, int seekMode)
     {
         if (ioMode == 0) { /* Read mode */
-            return Tcl_Seek (chan, offset, seekMode);
+            return Tcl_Seek(chan, offset, seekMode);
         } else {
             return fseek((FILE *)chan, offset, seekMode);
         }
@@ -257,9 +237,7 @@ static void isetname(IMAGE *image, const char *name)
     strncpy(image->name,name,80);
 }
 
-static void cvtshorts( buffer, n)
-unsigned short buffer[];
-int n;
+static void cvtshorts(unsigned short buffer[], int n)
 {
     short i;
     int nshorts = n>>1;
@@ -271,9 +249,7 @@ int n;
     }
 }
 
-static void cvtlongs( buffer, n)
-int buffer[];
-int n;
+static void cvtlongs(int buffer[], int n)
 {
     short i;
     int nlongs = n>>2;
@@ -290,8 +266,7 @@ int n;
     }
 }
 
-static void cvtimage( buffer )
-int buffer[];
+static void cvtimage(int buffer[])
 {
     cvtshorts((unsigned short *)buffer,12);
     cvtlongs(buffer+3,12);
@@ -315,7 +290,7 @@ static int imgOpenWrite (MYCHANNEL file, IMAGE *image, const char *mode,
                      unsigned int xsize, unsigned int ysize, unsigned int zsize)
 {
 #ifdef TCLSEEK_WORKAROUND
-        ioMode = 1;
+    ioMode = 1;
 #endif
     return imgopen (0, file, image, mode, type, dim, xsize, ysize, zsize);
 }
@@ -406,7 +381,7 @@ static int imgopen(int f, MYCHANNEL file, IMAGE *image, const char *mode,
                 return 0;
             }
             if(image->dorev) {
-                cvtlongs(image->rowstart,tablesize);
+                cvtlongs((int *)image->rowstart,tablesize);
             }
             if ((size_t)Tcl_Read (file, (char *)image->rowsize, tablesize) != tablesize) {
                 i_errhdlr("iopen: error on read of rowsize\n");
@@ -452,7 +427,7 @@ static int iclose(IMAGE *image)
             img_optseek(image, 512L);
             tablesize = image->ysize*image->zsize*sizeof(int);
             if(image->dorev) {
-                cvtlongs(image->rowstart,tablesize);
+                cvtlongs((int *)image->rowstart,tablesize);
             }
             if ((size_t)img_write(image,(char *)(image->rowstart),tablesize) != tablesize) {
                 i_errhdlr("iclose: error on write of rowstart\n");
@@ -1020,10 +995,12 @@ static void sgiClose (SGIFILE *tf)
     if (tf->blueScan)  ckfree ((char *)tf->blueScan);
     if (tf->matteScan) ckfree ((char *)tf->matteScan);
     if (tf->pixbuf)    ckfree ((char *)tf->pixbuf);
+    if (tf->scanline)  ckfree ((char *)tf->scanline);
+    iclose (&tf->th);
     return;
 }
 
-#define OUT Tcl_WriteChars (outChan, str, -1)
+#define OUTSTR Tcl_WriteChars (outChan, str, -1)
 static void printImgInfo (IMAGE *th, const char *filename, const char *msg)
 {
     Tcl_Channel outChan;
@@ -1033,14 +1010,14 @@ static void printImgInfo (IMAGE *th, const char *filename, const char *msg)
     if (!outChan) {
         return;
     }
-    sprintf(str, "%s %s\n", msg, filename);                                      OUT;
-    sprintf(str, "\tSize in pixel      : %d x %d\n", th->xsize, th->ysize);      OUT;
-    sprintf(str, "\tNo. of channels    : %d\n", (th->zsize));                    OUT;
-    sprintf(str, "\tBytes per pixel    : %d\n", BPP(th->type));                  OUT;
-    sprintf(str, "\tCompression        : %s\n", ISRLE(th->type)? "rle": "None"); OUT;
+    tkimg_snprintf(str, 256, "%s %s\n", msg, filename);                                      OUTSTR;
+    tkimg_snprintf(str, 256, "\tSize in pixel      : %d x %d\n", th->xsize, th->ysize);      OUTSTR;
+    tkimg_snprintf(str, 256, "\tNo. of channels    : %d\n", (th->zsize));                    OUTSTR;
+    tkimg_snprintf(str, 256, "\tBytes per pixel    : %d\n", BPP(th->type));                  OUTSTR;
+    tkimg_snprintf(str, 256, "\tCompression        : %s\n", ISRLE(th->type)? "rle": "None"); OUTSTR;
     Tcl_Flush(outChan);
 }
-#undef OUT
+#undef OUTSTR
 
 static Boln readHeader (tkimg_MFile *handle, IMAGE *th)
 {
@@ -1176,11 +1153,11 @@ static int CommonWrite(Tcl_Interp *interp,
         const char *filename, Tcl_Obj *format,
         tkimg_MFile *handle, Tk_PhotoImageBlock *blockPtr);
 
-static int ParseFormatOpts (interp, format, opts)
-    Tcl_Interp *interp;
-    Tcl_Obj *format;
-    FMTOPT *opts;
-{
+static int ParseFormatOpts(
+    Tcl_Interp *interp,
+    Tcl_Obj *format,
+    FMTOPT *opts
+) {
     static const char *const sgiOptions[] = {
         "-compression", "-verbose", "-matte", NULL
     };
@@ -1199,7 +1176,7 @@ static int ParseFormatOpts (interp, format, opts)
     }
     if (objc) {
         for (i=1; i<objc; i++) {
-            if (Tcl_GetIndexFromObj(interp, objv[i], (const char *CONST86 *)sgiOptions,
+            if (Tcl_GetIndexFromObj(interp, objv[i], (const char * const *)sgiOptions,
                     "format option", 0, &index) != TCL_OK) {
                 return TCL_ERROR;
             }
@@ -1271,18 +1248,19 @@ static int ObjMatch(
 ) {
     tkimg_MFile handle;
 
-    if (!tkimg_ReadInit(data, '\001', &handle)) {
+    if (! tkimg_ReadInit(data, '\xDA', &handle) &&
+        ! tkimg_ReadInit(data, '\001', &handle)) {
         return 0;
     }
     return CommonMatch(&handle, widthPtr, heightPtr, NULL);
 }
 
-static int CommonMatch(handle, widthPtr, heightPtr, sgiHeaderPtr)
-    tkimg_MFile *handle;
-    int *widthPtr;
-    int *heightPtr;
-    IMAGE *sgiHeaderPtr;
-{
+static int CommonMatch(
+    tkimg_MFile *handle,
+    int *widthPtr,
+    int *heightPtr,
+    IMAGE *sgiHeaderPtr
+) {
     IMAGE th;
 
     if (!sgiHeaderPtr) {
@@ -1307,17 +1285,16 @@ static int CommonMatch(handle, widthPtr, heightPtr, sgiHeaderPtr)
     return 1;
 }
 
-static int ChnRead(interp, chan, filename, format, imageHandle,
-                   destX, destY, width, height, srcX, srcY)
-    Tcl_Interp *interp;
-    Tcl_Channel chan;
-    const char *filename;
-    Tcl_Obj *format;
-    Tk_PhotoHandle imageHandle;
-    int destX, destY;
-    int width, height;
-    int srcX, srcY;
-{
+static int ChnRead(
+    Tcl_Interp *interp,
+    Tcl_Channel chan,
+    const char *filename,
+    Tcl_Obj *format,
+    Tk_PhotoHandle imageHandle,
+    int destX, int destY,
+    int width, int height,
+    int srcX, int srcY
+) {
     tkimg_MFile handle;
 
     handle.data = (char *) chan;
@@ -1330,50 +1307,81 @@ static int ChnRead(interp, chan, filename, format, imageHandle,
 
 #define BUFLEN 2048
 
-static int ObjRead (interp, data, format, imageHandle,
-                    destX, destY, width, height, srcX, srcY)
-    Tcl_Interp *interp;
-    Tcl_Obj *data;
-    Tcl_Obj *format;
-    Tk_PhotoHandle imageHandle;
-    int destX, destY;
-    int width, height;
-    int srcX, srcY;
-{
+static int ObjRead(
+    Tcl_Interp *interp,
+    Tcl_Obj *data,
+    Tcl_Obj *format,
+    Tk_PhotoHandle imageHandle,
+    int destX, int destY,
+    int width, int height,
+    int srcX, int srcY
+) {
     tkimg_MFile handle;
-    char tempFileNameBuffer[1024];
+    char *dir, *tempFileName, tempFileNameBuffer[1024];
+#ifdef WIN32
+    char tempPath[512];
+    HANDLE h;
+#endif
+    Tcl_DString ds;
     char buffer[BUFLEN];
-    MYCHANNEL outchan;
+    FILE *outfile;
     Tcl_Channel inchan;
     int count, retVal;
 
-    tkimg_ReadInit (data, '\001', &handle);
-
-    if (! TempFileName(tempFileNameBuffer)) {
+    if (! tkimg_ReadInit(data, '\xDA', &handle) &&
+        ! tkimg_ReadInit(data, '\001', &handle)) {
         return TCL_ERROR;
     }
-#ifdef TCLSEEK_WORKAROUND
-    outchan = (Tcl_Channel)fopen (tempFileNameBuffer, "wb");
+
+    tempFileName = tempFileNameBuffer;
+#ifdef WIN32
+    strcpy(tempPath, ".");
+    GetTempPathA(sizeof (tempPath), tempPath);
+    dir = tempPath;
+    tempFileName[0] = '\0';
+    GetTempFileNameA(dir, "tki", 0, tempFileName);
+    h = CreateFileA(tempFileName, GENERIC_READ|GENERIC_WRITE, 0, NULL,
+            CREATE_ALWAYS, FILE_ATTRIBUTE_TEMPORARY, NULL);
+    if (h != INVALID_HANDLE_VALUE) {
+        CloseHandle(h);
+    }
 #else
-    outchan = tkimg_OpenFileChannel (interp, tempFileNameBuffer, 0644);
+    dir = getenv("TMPDIR");
+    if (dir) {
+        strcpy(tempFileName, dir);
+    } else {
+#ifdef P_tmpdir
+        strcpy(tempFileName, P_tmpdir);
+#else
+        strcpy(tempFilename, "/tmp");
 #endif
-    if (!outchan) {
+    }
+    strcat(tempFileName, "/tkimgXXXXXX");
+    retVal = mkstemp(tempFileName);
+    if (retVal >= 0) {
+        close(retVal);
+    }
+#endif
+
+    outfile = fopen(tempFileName, "wb");
+    if (outfile == NULL) {
+        Tcl_AppendResult(interp, "error open output file", (char *) NULL);
         return TCL_ERROR;
     }
 
     count = tkimg_Read2(&handle, buffer, BUFLEN);
     while (count == BUFLEN) {
-        Tcl_Write (outchan, buffer, count);
+        fwrite(buffer, 1, count, outfile);
         count = tkimg_Read2(&handle, buffer, BUFLEN);
     }
     if (count>0) {
-        Tcl_Write (outchan, buffer, count);
-    }
-    if (MYCLOSE (interp, outchan) == TCL_ERROR) {
-        return TCL_ERROR;
+        fwrite(buffer, 1, count, outfile);
     }
+    fclose(outfile);
 
-    inchan = tkimg_OpenFileChannel (interp, tempFileNameBuffer, 0);
+    Tcl_ExternalToUtfDString(NULL, tempFileName, -1, &ds);
+    inchan = tkimg_OpenFileChannel(interp, Tcl_DStringValue(&ds), 0);
+    Tcl_DStringFree(&ds);
     if (!inchan) {
         return TCL_ERROR;
     }
@@ -1386,24 +1394,27 @@ static int ObjRead (interp, data, format, imageHandle,
     if (Tcl_Close (interp, inchan) == TCL_ERROR) {
         return TCL_ERROR;
     }
-    remove (tempFileNameBuffer);
+#ifdef WIN32
+    DeleteFileA(tempFileName);
+#else
+    remove(tempFileName);
+#endif
     return retVal;
 }
 
-static int CommonRead (interp, handle, filename, format, imageHandle,
-                       destX, destY, width, height, srcX, srcY)
-    Tcl_Interp *interp;         /* Interpreter to use for reporting errors. */
-    tkimg_MFile *handle;        /* The image file, open for reading. */
-    const char *filename;       /* The name of the image file. */
-    Tcl_Obj *format;            /* User-specified format object, or NULL. */
-    Tk_PhotoHandle imageHandle; /* The photo image to write into. */
-    int destX, destY;           /* Coordinates of top-left pixel in
+static int CommonRead(
+    Tcl_Interp *interp,         /* Interpreter to use for reporting errors. */
+    tkimg_MFile *handle,        /* The image file, open for reading. */
+    const char *filename,       /* The name of the image file. */
+    Tcl_Obj *format,            /* User-specified format object, or NULL. */
+    Tk_PhotoHandle imageHandle, /* The photo image to write into. */
+    int destX, int destY,       /* Coordinates of top-left pixel in
                                  * photo image to be written to. */
-    int width, height;          /* Dimensions of block of photo image to
+    int width, int height,      /* Dimensions of block of photo image to
                                  * be written to. */
-    int srcX, srcY;             /* Coordinates of top-left pixel to be used
+    int srcX, int srcY          /* Coordinates of top-left pixel to be used
                                  * in image being read. */
-{
+) {
     Tk_PhotoImageBlock block;
     Int y, nchan;
     int fileWidth, fileHeight;
@@ -1411,6 +1422,7 @@ static int CommonRead (interp, handle, filename, format, imageHandle,
     SGIFILE tf;
     FMTOPT opts;
     int result = TCL_OK;
+    char msgStr[1024];
 
     memset(&tf, 0, sizeof (SGIFILE));
     if (ParseFormatOpts(interp, format, &opts) != TCL_OK) {
@@ -1434,7 +1446,8 @@ static int CommonRead (interp, handle, filename, format, imageHandle,
     }
     if ((outWidth <= 0) || (outHeight <= 0)
         || (srcX >= fileWidth) || (srcY >= fileHeight)) {
-        return TCL_OK;
+        Tcl_AppendResult(interp, "Width or height are negative", (char *) NULL);
+        return TCL_ERROR;
     }
 
     if (tkimg_PhotoExpand(interp, imageHandle, destX + outWidth, destY + outHeight) == TCL_ERROR) {
@@ -1443,8 +1456,17 @@ static int CommonRead (interp, handle, filename, format, imageHandle,
 
     nchan = tf.th.zsize;
 
-    tf.pixbuf   = (UShort *) ckalloc (fileWidth * nchan * sizeof (UShort));
-    tf.scanline = (UByte  *) ckalloc (fileWidth * nchan);
+    tf.pixbuf   = (UShort *) attemptckalloc (fileWidth * nchan * sizeof (UShort));
+    if (tf.pixbuf == NULL) {
+        Tcl_AppendResult (interp, "Unable to allocate memory for image data.", (char *) NULL);
+        return TCL_ERROR;
+    }
+    tf.scanline = (UByte  *) attemptckalloc (fileWidth * nchan);
+    if (tf.scanline == NULL) {
+        ckfree( (char *) tf.pixbuf);
+        Tcl_AppendResult (interp, "Unable to allocate memory for image data.", (char *) NULL);
+        return TCL_ERROR;
+    }
 
     block.pixelSize = nchan;
     block.pitch = fileWidth * nchan;
@@ -1466,7 +1488,8 @@ static int CommonRead (interp, handle, filename, format, imageHandle,
             block.offset[3] = opts.matte? 3: 0;
             break;
         default:
-            printf("Invalid number of channels: %d\n", (int) nchan);
+            tkimg_snprintf(msgStr, 1024, "Invalid number of channels: %d", nchan);
+            Tcl_AppendResult(interp, msgStr, (char *)NULL);
             return TCL_ERROR;
             break;
     }
@@ -1490,20 +1513,24 @@ static int CommonRead (interp, handle, filename, format, imageHandle,
     return result;
 }
 
-static int ChnWrite (interp, filename, format, blockPtr)
-    Tcl_Interp *interp;
-    const char *filename;
-    Tcl_Obj *format;
-    Tk_PhotoImageBlock *blockPtr;
-{
+static int ChnWrite(
+    Tcl_Interp *interp,
+    const char *filename,
+    Tcl_Obj *format,
+    Tk_PhotoImageBlock *blockPtr
+) {
     MYCHANNEL chan;
     tkimg_MFile handle;
     int result;
 
 #ifdef TCLSEEK_WORKAROUND
-    chan = (Tcl_Channel)fopen(filename, "wb");
+    Tcl_DString ds;
+
+    Tcl_UtfToExternalDString(NULL, filename, -1, &ds);
+    chan = (Tcl_Channel)fopen(Tcl_DStringValue(&ds), "wb");
+    Tcl_DStringFree(&ds);
 #else
-    chan = tkimg_OpenFileChannel (interp, filename, 0644);
+    chan = tkimg_OpenFileChannel(interp, filename, 0644);
 #endif
     if (!chan) {
         return TCL_ERROR;
@@ -1526,37 +1553,73 @@ static int StringWrite(
 ) {
     tkimg_MFile handle;
     int result;
-    Tcl_DString data;
+    Tcl_DString data, ds;
     Tcl_Channel inchan;
     MYCHANNEL outchan;
-    char tempFileNameBuffer[1024];
+    char *dir, *tempFileName, tempFileNameBuffer[1024];
+#ifdef WIN32
+    char tempPath[512];
+    HANDLE h;
+#endif
     char buffer[BUFLEN];
     int count;
 
     Tcl_DStringInit(&data);
-    if (! TempFileName(tempFileNameBuffer)) {
-        return TCL_ERROR;
+    tempFileName = tempFileNameBuffer;
+#ifdef WIN32
+    strcpy(tempPath, ".");
+    GetTempPathA(sizeof (tempPath), tempPath);
+    dir = tempPath;
+    tempFileName[0] = '\0';
+    GetTempFileNameA(dir, "tki", 0, tempFileName);
+    h = CreateFileA(tempFileName, GENERIC_READ|GENERIC_WRITE, 0, NULL,
+            CREATE_ALWAYS, FILE_ATTRIBUTE_TEMPORARY, NULL);
+    if (h != INVALID_HANDLE_VALUE) {
+        CloseHandle(h);
     }
-#ifdef TCLSEEK_WORKAROUND
-    outchan = (Tcl_Channel)fopen(tempFileNameBuffer, "wb");
 #else
-    outchan = tkimg_OpenFileChannel (interp, tempFileNameBuffer, 0644);
+    dir = getenv("TMPDIR");
+    if (dir) {
+        strcpy(tempFileName, dir);
+    } else {
+#ifdef P_tmpdir
+        strcpy(tempFileName, P_tmpdir);
+#else
+        strcpy(tempFilename, "/tmp");
+#endif
+    }
+    strcat(tempFileName, "/tkimgXXXXXX");
+    result = mkstemp(tempFileName);
+    if (result >= 0) {
+        close(result);
+    }
+#endif
+    Tcl_ExternalToUtfDString(NULL, tempFileName, -1, &ds);
+#ifdef TCLSEEK_WORKAROUND
+    outchan = (Tcl_Channel)fopen(tempFileName, "wb");
+#else
+    outchan = tkimg_OpenFileChannel(interp, Tcl_DStringValue(&ds), 0644);
 #endif
     if (!outchan) {
+        Tcl_DStringFree(&ds);
         return TCL_ERROR;
     }
 
     handle.data = (char *) outchan;
     handle.state = IMG_CHAN;
 
-    result = CommonWrite(interp, tempFileNameBuffer, format, &handle, blockPtr);
+    result =
+        CommonWrite(interp, Tcl_DStringValue(&ds), format, &handle, blockPtr);
+    Tcl_DStringFree(&ds);
     if (MYCLOSE(interp, outchan) == TCL_ERROR) {
         return TCL_ERROR;
     }
 
     tkimg_WriteInit(&data, &handle);
 
-    inchan = tkimg_OpenFileChannel(interp, tempFileNameBuffer, 0);
+    Tcl_ExternalToUtfDString(NULL, tempFileName, -1, &ds);
+    inchan = tkimg_OpenFileChannel(interp, Tcl_DStringValue(&ds), 0);
+    Tcl_DStringFree(&ds);
     if (!inchan) {
         return TCL_ERROR;
     }
@@ -1572,7 +1635,11 @@ static int StringWrite(
     if (Tcl_Close(interp, inchan) == TCL_ERROR) {
         return TCL_ERROR;
     }
-    remove (tempFileNameBuffer);
+#ifdef WIN32
+    DeleteFileA(tempFileName);
+#else
+    remove(tempFileName);
+#endif
     tkimg_Putc(IMG_DONE, &handle);
 
     if (result == TCL_OK) {
@@ -1583,13 +1650,13 @@ static int StringWrite(
     return result;
 }
 
-static int CommonWrite (interp, filename, format, handle, blockPtr)
-    Tcl_Interp *interp;
-    const char *filename;
-    Tcl_Obj *format;
-    tkimg_MFile *handle;
-    Tk_PhotoImageBlock *blockPtr;
-{
+static int CommonWrite(
+    Tcl_Interp *interp,
+    const char *filename,
+    Tcl_Obj *format,
+    tkimg_MFile *handle,
+    Tk_PhotoImageBlock *blockPtr
+) {
     Int     x, y, bpp, nchan;
     Int     redOffset, greenOffset, blueOffset, alphaOffset;
     UByte   *pixelPtr, *rowPixPtr;
@@ -1618,11 +1685,16 @@ static int CommonWrite (interp, filename, format, handle, blockPtr)
     }
     nchan = ((opts.matte && alphaOffset)? 4: 3);
 
-    tf.redScan   = (UByte *)  ckalloc (blockPtr->width);
-    tf.greenScan = (UByte *)  ckalloc (blockPtr->width);
-    tf.blueScan  = (UByte *)  ckalloc (blockPtr->width);
-    tf.matteScan = (UByte *)  ckalloc (blockPtr->width);
-    tf.pixbuf    = (UShort *) ckalloc (blockPtr->width * sizeof (UShort));
+    tf.redScan   = (UByte *)  attemptckalloc (blockPtr->width);
+    tf.greenScan = (UByte *)  attemptckalloc (blockPtr->width);
+    tf.blueScan  = (UByte *)  attemptckalloc (blockPtr->width);
+    tf.matteScan = (UByte *)  attemptckalloc (blockPtr->width);
+    tf.pixbuf    = (UShort *) attemptckalloc (blockPtr->width * sizeof (UShort));
+    if (tf.redScan == NULL || tf.greenScan == NULL || tf.blueScan == NULL || tf.matteScan == NULL || tf.pixbuf == NULL) {
+        sgiClose( &tf );
+        Tcl_AppendResult (interp, "Unable to allocate memory for image data.", (char *) NULL);
+        return TCL_ERROR;
+    }
     tf.th.imagic = IMAGIC;
 
     if (!writeHeader(handle, &tf.th,
@@ -1660,7 +1732,6 @@ static int CommonWrite (interp, filename, format, handle, blockPtr)
         printImgInfo (&tf.th, filename, "Saving image:");
     }
 
-    iclose (&tf.th);
     sgiClose (&tf);
     return TCL_OK;
 }
