@@ -141,9 +141,9 @@ getint(
             result = (buf[order]<<8) + buf[1-order]; break;
         case TIFF_LONG:
             if (order) {
-                result = (buf[3]<<24) + (buf[2]<<16) + (buf[1]<<8) + buf[0];
+                result = ((unsigned int)buf[3]<<24) + (buf[2]<<16) + (buf[1]<<8) + buf[0];
             } else {
-                result = (buf[0]<<24) + (buf[1]<<16) + (buf[2]<<8) + buf[3];
+                result = ((unsigned int)buf[0]<<24) + (buf[1]<<16) + (buf[2]<<8) + buf[3];
             }; break;
         default:
             result = -1;
@@ -161,16 +161,14 @@ _TIFFerr(
     char *cp = buf;
 
     if (module != NULL) {
-        sprintf(cp, "%s: ", module);
+        tkimg_snprintf(cp, 2048, "%s: ", module);
         cp += strlen(module) + 2;
     }
 
-    vsprintf(cp, fmt, ap);
+    tkimg_vsnprintf(cp, 2048 - (cp - buf), fmt, ap);
     if (errorMessage) {
         ckfree(errorMessage);
-    }
-    if (strstr(buf, "Null count for")) {
-        return;
+        errorMessage = NULL;
     }
     errorMessage = (char *) ckalloc(strlen(buf)+1);
     strcpy(errorMessage, buf);
@@ -263,7 +261,12 @@ readString(
     tkimg_MFile *handle = (tkimg_MFile *) fd;
 
     if (((size_t)size + handle->state) > handle->length) {
-        size = handle->length - handle->state;
+        /* Avoid unsigned underflow. */
+        if (handle->state > handle->length) {
+            size = 0;
+        } else {
+            size = handle->length - handle->state;
+        }
     }
     if (size) {
         memcpy((char *) data, handle->data + handle->state, (size_t) size);
@@ -429,8 +432,8 @@ ObjRead(
     Tcl_Obj *data,                      /* object containing the image */
     Tcl_Obj *format,
     Tk_PhotoHandle imageHandle,
-    int destX, int destY, 
-    int width, int height, 
+    int destX, int destY,
+    int width, int height,
     int srcX, int srcY
 ) {
     TIFF *tif;
@@ -446,7 +449,11 @@ ObjRead(
 
     if (TIFFClientOpen) {
         if (handle.state != IMG_STRING) {
-            dataPtr = ckalloc((handle.length*3)/4 + 2);
+            dataPtr = attemptckalloc((handle.length*3)/4 + 2);
+            if (dataPtr == NULL) {
+                Tcl_AppendResult (interp, "Unable to allocate memory for image data.", (char *) NULL);
+                return TCL_ERROR;
+            }
             handle.length = tkimg_Read2(&handle, dataPtr, handle.length);
             handle.data = dataPtr;
         }
@@ -510,6 +517,7 @@ ObjRead(
     if (tif != NULL) {
         result = CommonRead(interp, tif, format, imageHandle,
                  destX, destY, width, height, srcX, srcY);
+        TIFFClose(tif);
     } else {
         result = TCL_ERROR;
     }
@@ -517,9 +525,11 @@ ObjRead(
         unlink(tempFileName);
     }
     if (result == TCL_ERROR) {
-        Tcl_AppendResult(interp, errorMessage, (char *) NULL);
-        ckfree(errorMessage);
-        errorMessage = NULL;
+        if (strlen (Tcl_GetStringResult(interp)) == 0 && errorMessage) {
+            Tcl_AppendResult(interp, errorMessage, (char *) NULL);
+            ckfree(errorMessage);
+            errorMessage = NULL;
+        }
     }
     if (dataPtr) {
         ckfree(dataPtr);
@@ -541,10 +551,10 @@ ChnRead(
     TIFF *tif;
     char *dir, *tempFileName = NULL, tempFileNameBuffer[1024];
     int count, result;
+    tkimg_MFile handle;
     char buffer[4096];
 
     if (TIFFClientOpen) {
-        tkimg_MFile handle;
         handle.data = (char *) chan;
         handle.state = IMG_CHAN;
         tif = TIFFClientOpen(fileName, "r", (thandle_t) &handle,
@@ -606,6 +616,7 @@ ChnRead(
     if (tif) {
         result = CommonRead(interp, tif, format, imageHandle,
                 destX, destY, width, height, srcX, srcY);
+        TIFFClose(tif);
     } else {
         result = TCL_ERROR;
     }
@@ -613,9 +624,11 @@ ChnRead(
         unlink(tempFileName);
     }
     if (result == TCL_ERROR) {
-        Tcl_AppendResult(interp, errorMessage, (char *) NULL);
-        ckfree(errorMessage);
-        errorMessage = 0;
+        if (strlen (Tcl_GetStringResult(interp)) == 0 && errorMessage) {
+            Tcl_AppendResult(interp, errorMessage, (char *) NULL);
+            ckfree(errorMessage);
+            errorMessage = NULL;
+        }
     }
     return result;
 }
@@ -631,10 +644,9 @@ CommonRead(
     int srcX, int srcY
 ) {
     Tk_PhotoImageBlock block;
-    uint32 w, h;
+    uint32_t w, h;
     size_t npixels;
-    uint32 *raster;
-    int result = TCL_OK;
+    uint32_t *raster;
     int nBytes, index = 0, objc = 0;
     Tcl_Obj **objv = NULL;
 
@@ -671,29 +683,33 @@ CommonRead(
     block.offset[2] = 2;
     block.offset[3] = 3;
 #endif
-    block.pixelSize = sizeof (uint32);
+    block.pixelSize = sizeof (uint32_t);
 
     TIFFGetField(tif, TIFFTAG_IMAGEWIDTH,  &w);
     TIFFGetField(tif, TIFFTAG_IMAGELENGTH, &h);
-    npixels = w * h;
+    npixels = (size_t)w * (size_t)h;
 
-    raster = (uint32*) TkimgTIFFmalloc(npixels * sizeof (uint32));
+    if (npixels * sizeof (uint32_t) >= UINT_MAX) {
+        Tcl_AppendResult(interp, "Image size too large", (char *) NULL);
+        return TCL_ERROR;
+    }
+    raster = (uint32_t*) TkimgTIFFmalloc(npixels * sizeof (uint32_t));
+    if (raster == NULL) {
+        Tcl_AppendResult(interp, "Cannot allocate raster memory", (char *) NULL);
+        return TCL_ERROR;
+    }
     block.width = w;
     block.height = h;
     block.pitch = - (block.pixelSize * (int) w);
     block.pixelPtr = ((unsigned char *) raster) + ((1-h) * block.pitch);
-    if (raster == NULL) {
-        printf("cannot malloc\n");
-        return TCL_ERROR;
-    }
 
     if (!TIFFReadRGBAImage(tif, w, h, raster, 0) || errorMessage) {
-        TkimgTIFFfree (raster);
         if (errorMessage) {
             Tcl_AppendResult(interp, errorMessage, (char *) NULL);
             ckfree(errorMessage);
             errorMessage = NULL;
         }
+        TkimgTIFFfree (raster);
         return TCL_ERROR;
     }
 
@@ -701,12 +717,12 @@ CommonRead(
     block.offset[3] = block.offset[0]; /* don't use transparency */
     if (tkimg_PhotoPutBlock(interp, imageHandle, &block, destX,
                         destY, width, height, TK_PHOTO_COMPOSITE_SET) == TCL_ERROR) {
-        result = TCL_ERROR;
+        TkimgTIFFfree (raster);
+        return TCL_ERROR;
     }
 
     TkimgTIFFfree (raster);
-    TIFFClose(tif);
-    return result;
+    return TCL_OK;
 }
 
 static int StringWrite(
@@ -887,7 +903,7 @@ ParseWriteFormat(
         compression = "none";
         byteorder = "";
         for (i=1; i<objc; i++) {
-            if (Tcl_GetIndexFromObj(interp, objv[i], (const char *CONST86 *)tiffWriteOptions,
+            if (Tcl_GetIndexFromObj(interp, objv[i], (const char * const *)tiffWriteOptions,
                     "format option", 0, &index) !=TCL_OK) {
                 return TCL_ERROR;
             }
@@ -987,8 +1003,12 @@ CommonWrite(
     } else {
         unsigned char *srcPtr, *dstPtr, *rowPtr;
         int greenOffset, blueOffset, alphaOffset, x, y;
-        dstPtr = data = (unsigned char *) ckalloc(numsamples *
+        dstPtr = data = (unsigned char *) attemptckalloc(numsamples *
                 blockPtr->width * blockPtr->height);
+        if (dstPtr == NULL) {
+            Tcl_AppendResult (interp, "Unable to allocate memory for image data.", (char *) NULL);
+            return TCL_ERROR;
+        }
         rowPtr = blockPtr->pixelPtr + blockPtr->offset[0];
         greenOffset = blockPtr->offset[1] - blockPtr->offset[0];
         blueOffset = blockPtr->offset[2] - blockPtr->offset[0];
